/******
 * ASMCDict - Dictionary creator, belongs to ASMCrack
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 */


/****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <dir.h>
#include <signal.h>
/****************************************************************************/


/****************************************************************************/
#define VERSION_STR "2.56"     /* current version */

#define MAXRAMWORDS 200000     /* Maximum number of words in RAM */
#define MAXWORDS    16000000   /* Maximum number of words */
#define CACHESIZE   131072     /* size of hard disk cache (must be 2^n) */
#define MAXCACHEWS  1024       /* maximum number of cache entries that */
                               /* are written at once */
#define NOINDEX     0xFFFFFF   /* this number marks non existing indexes */

#define isvowel(c) (strchr ("aeiou", c) != NULL)
#define ABS(v) ((v) > 0 ? (v) : -(v))
/****************************************************************************/


/**
 * Dict[MAXRAMWORDS] is a binary tree (see DICTIONARY_TREE - structure)
 * that controlls itself to grow in all directions by calculating the
 * perfect value for a junction... (see AddWord)
 **
 * If the number of words overflows MAXRAMWORDS, hard disk swapping will be
 * enabled. This means, that all entries beyond MAXRAMWORDS are expected to
 * be in the file pointed by SwapHdl in a linear order. The temporary
 * file is accessed, using a simple cache (see CACHE_ENTRY), that can hold
 * CACHSIZE dictionary entries.
 */

/****************************************************************************/
/* binary tree for the words */
typedef struct  DICTIONARY
{
  long long Str;
  unsigned  Upper : 24;
  unsigned  Lower : 24;
  char      Killed;
} DICTIONARY;

/* element of the cache */
typedef struct CACHEINFO
{
  char      Modified;
  unsigned  I : 24;
} CACHEINFO;

/****************************************************************************/


/****************************************************************************/
DICTIONARY RAMDict[MAXRAMWORDS], /* RAM dictionary */
           Cache[CACHESIZE];     /* Hard disk swapping cache - contents */
CACHEINFO  CacheInf[CACHESIZE];  /* Hard disk swapping cache - information */



long      Words = 0,             /* number of words */
          KilledWords = 0;       /* number of removed words */

int       MaxLength      = 200,  /* Maximum length of word to read it */
          MinLength      = 2,    /* Minimum length of word to read it */
          TruncLength    = 8,    /* Length to cut the word down to */
          IncludeOrigin  = 0,    /* Include original words */
          Lowercase[8],          /* Convert first x chars of word to lowercase */
          Uppercase[8],          /* Convert first x chars of word to uppercase */
          LowConvNum     = 0,    /* Number of lowercase conversions */
          UppConvNum     = 0,    /* Number of uppercase conversions */
          Reverse        = 0,    /* Add reversed words to dict */
          NoVowels       = 0,    /* Add vowel-eliminated words to dict */
          Alpha          = 0,    /* Only read words that consist of A-Z,a-z */
          WholeWords     = 0,    /* read whole words only */
          IBuf           = 0,    /* Index of current word in Buffer */
          SwapHdl;               /* Handle of swapping file */
char      Suffixes[256]  = "",
          Prefixes[256]  = "",
          OnlyChars[256] = "",   /* Only read words that consist of these chars */
          Replace[256] = "",    /* String: XY - X is replaced by Y */
          SwapName[256] = "",    /* Name of swapping file */
          OutName[256] = "",     /* Output file name */
          SubName[256] = "";     /* Name of dictionary that is substracted */

/****************************************************************************/


/**
 * Swap file functions
 */

/** Create hard disk dictionary */
void CreateSwap ()
{
  register int I;

  strcpy (SwapName, tmpnam (NULL));
  if ((SwapHdl = open (SwapName, O_BINARY|O_RDWR|O_CREAT|O_TRUNC)) < 0)
  {
    perror ("\nCan't create temporary file");
    exit (1);
  }

  for (I = 0; I < CACHESIZE; I++)
  {
    CacheInf[I].I = NOINDEX;
    CacheInf[I].Modified = 0;
  }
}

/** Remove hard disk dictionary */
void CloseSwap ()
{
  close (SwapHdl);
  remove (SwapName);
}

/** writes n zeroes to the file specified by hdl */
void WriteZeroes (int hdl, long n)
{
  static char dummy[1024];

  while (n > 0)
    n -= write (hdl, dummy, n > 1024 ? 1024 : n);
}

/** write entries from the cache to the swapping file */
__inline__ void WriteSwap (long i, long ci)
{
  int I;
  long is, should;


  for (I = 1; I < MAXCACHEWS && ci+I < CACHESIZE &&
              CacheInf[I+ci].I == I+i && CacheInf[I+ci].Modified; I++)
    CacheInf[I+ci].Modified = 0;

  /* if the given index is beyond the file-end -> extend the file */
  should = i*sizeof (DICTIONARY);
  if ((is = lseek (SwapHdl, should, SEEK_SET)) < should);
    WriteZeroes (SwapHdl, should-is);

  if (write (SwapHdl, &Cache[ci], sizeof (DICTIONARY)*I) == -1)
  {
    perror ("\nTemporary file error");
    exit (1);
  }
}

/** read one entry from the swapping file */
__inline__ DICTIONARY *ReadSwap (long i, long ci)
{
  lseek (SwapHdl, i * sizeof (DICTIONARY), SEEK_SET);

  if (read (SwapHdl, &Cache[ci], sizeof (DICTIONARY)) == -1)
  {
    perror ("\nTemporary file error");
    exit (1);
  }

  return (&Cache[ci]);
}

/** get element of hard disk dictionary */
__inline__ DICTIONARY *getSwapEntry (long i)
{
  long  ci;

  /* entry in cache yet? */
  if (CacheInf[ci = i & (CACHESIZE-1)].I == i)
    return (&Cache[ci]);

  if (CacheInf[ci].Modified)
    WriteSwap (CacheInf[ci].I, ci);

  CacheInf[ci].I = i;
  CacheInf[ci].Modified = 0;

  return (ReadSwap (i, ci));
}

/** get element of hard disk dictionary and mark it to be modified */
__inline__ DICTIONARY *ModifySwapEntry (long i)
{
  long  ci;

  /* entry in cache yet? */
  if (CacheInf[ci = i & (CACHESIZE-1)].I == i)
  {
    CacheInf[ci].Modified = 1;
    return (&Cache[ci]);
  }

  if (CacheInf[ci].Modified)
    WriteSwap (CacheInf[ci].I, ci);

  CacheInf[ci].I = i;

  return (ReadSwap (i, ci));
}

/** create a new swap entry */
__inline__ DICTIONARY *newSwapEntry (int i)
{
  long  ci;

  if (CacheInf[ci = i & (CACHESIZE-1)].Modified)
    WriteSwap (CacheInf[ci].I, ci);

  CacheInf[ci].Modified = 1;
  CacheInf[ci].I = i;

  return (&Cache[ci]);
}


/**
 * Dictionary access functions
 */
/** get index of a new word entry, and initialize this entry */
long NewWord (long long word)
{
  static DICTIONARY *e;

  if (Words == MAXWORDS)
    return (Words-1);

  if (Words < MAXRAMWORDS)
  {
    RAMDict[Words].Lower = RAMDict[Words].Upper = NOINDEX;
    RAMDict[Words].Killed = 0;
    RAMDict[Words].Str = word;
  }
  else
  {
    if (Words == MAXRAMWORDS)
      fputs ("\nOut of memory - harddisk swapping enabled.\n", stderr);

    e = newSwapEntry (Words-MAXRAMWORDS);
    e->Lower = e->Upper = NOINDEX;
    e->Killed = 0;
    e->Str = word;
  }

  return (Words++);
}

/** get Dictionary[i].Upper */
__inline__ long getDictUpper (long i)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Upper);
  return (getSwapEntry (i-MAXRAMWORDS)->Upper);
}

/** get Dictionary[i].Lower */
__inline__ long getDictLower (long i)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Lower);
  return (getSwapEntry (i-MAXRAMWORDS)->Lower);
}

/** get Dictionary[i].Lower */
__inline__ long getDictKilled (long i)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Killed);
  return (getSwapEntry (i-MAXRAMWORDS)->Killed);
}

/** get Dictionary[i].Str */
__inline__ long long getDictStr (long i)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Str);
  return (getSwapEntry (i-MAXRAMWORDS)->Str);
}

/** set Dictionary[i].Upper */
__inline__ long setDictUpper (long i, long v)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Upper = v);
  return (ModifySwapEntry (i-MAXRAMWORDS)->Upper = v);
}

/** set Dictionary[i].Lower */
__inline__ long setDictLower (long i, long v)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Lower = v);
  return (ModifySwapEntry (i-MAXRAMWORDS)->Lower = v);
}

/** set Dictionary[i].Lower */
__inline__ long setDictKilled (long i, long v)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Killed = v);
  return (ModifySwapEntry (i-MAXRAMWORDS)->Lower = v);
}

/** set Dictionary[i].Str */
__inline__ long long setDictStr (long i, long long v)
{
  if (i < MAXRAMWORDS)
    return (RAMDict[i].Str = v);
  return (ModifySwapEntry (i-MAXRAMWORDS)->Str = v);
}


/**
 * Returns whether word is a word using only 7bit chars
 */
int Is7bitWord (char *word)
{
  register int c;

  while (c = *word++)
    if (c & 128)
      return (0);

  return (1);
}

/**
 * Adds a word to the dictionary, that is a binary tree.
 * It won't be added if it is allready in there.
 */
void AddWord (char *str)
{
  long long perfect,    /* perfect mid element for a junction */
            perfchg,
            word,
            thisword,
            xword;
  long      I, J;
  char      *wordptr;

  if (!Is7bitWord (str) || !str[0])
    return;

  /* reverse byte order; raw => intel */
  for (I = 7, wordptr = (char *)&word, word = 0; *str && I >= 0; I--)
    wordptr[I] = *str++;

  if (!Words)
  {
    NewWord (word);
    return;
  }

  for (I = 0, perfect = 0x4000000000000000LL, perfchg = perfect >> 1;; perfchg >>= 1)
  {
    if ((thisword = getDictStr (I)) == word)
      break;

    /* new word more perfect for junction than the current word? */
    if (ABS (word-perfect) < ABS (thisword-perfect))
    {
      /* exchange this word with the new word */
      xword = setDictStr (I, word);
      word = thisword;
      thisword = xword;
    }

    if (word > thisword)
    {
      if ((I = getDictUpper (J = I)) == NOINDEX)
      {
        setDictUpper (J, NewWord (word));
        break;
      }
      perfect += perfchg;
    }
    else
    {
      if ((I = getDictLower (J = I)) == NOINDEX)
      {
        setDictLower (J, NewWord (word));
        break;
      }
      perfect -= perfchg;
    }
  }
}


/**
 * Removes a word from the dictionary, by setting the Killed flag of
 * the entry.
 */
void KillWord (char *str)
{
  long long word,
            thisword;
  long      I;
  char      *wordptr;

  if (!Is7bitWord (str) || !str[0] || !Words)
    return;

  /* reverse byte order; raw => intel */
  for (I = 7, wordptr = (char *)&word, word = 0; *str && I >= 0; I--)
    wordptr[I] = *str++;

  /* seek through binary tree */
  for (I = 0;;)
  {
    if ((thisword = getDictStr (I)) == word)
    {
      setDictKilled (I, 1);
      KilledWords++;
      break;
    }

    if (word > thisword)
    {
      if ((I = getDictUpper (I)) == NOINDEX)
        break;
    }
    else
      if ((I = getDictLower (I)) == NOINDEX)
        break;
  }
}


/**
 * Combines the word with all given suffixes, and add all the resulting
 * words to the dictionary
 */
void AddWordsSuffixes (char *word, int wlen)
{
  int I,
      c;
  for (I = 0, word[wlen+1] = '\0'; c = Suffixes[I]; I++)
  {
    word[wlen] = c;
    AddWord (word);
  }
  word[wlen] = '\0';
}


/**
 * Combines the word with all given prefixes, and add all the resulting
 * words to the dictionary
 */
void AddWordsPrefixes (char *word, int wlen)
{
  int I,
      c;

  memmove (&word[1], &word[0], wlen);

  for (I = 0, word[wlen+1] = '\0'; c = Prefixes[I]; I++)
  {
    word[0] = c;
    AddWord (word);
  }
}


/**
 * Reverses src, and stores the result in dest
 */
char *StrReverse (char *dest, char *src)
{
  register int I = 0, c;

  *dest = '\0';

  while (I++ < 8 && (c = *src++))
    *--dest = c;

  return (dest);
}


/**
 * Copys src to dest, removing all vowels
 */
int StrKillVowels (char *dest, char *src)
{
  register int I, c;

  for (I = 0; (c = *src++) != '\0';)
    if (!isvowel (c))
      dest[I++] = c;

  dest[I] = '\0';

  return (I);
}

/**
 * Converts the first >Uppercase< chars in src to uppercase, and stores the
 * result in dest
 */
char *StrUppConv (char *dest, char *src, int n)
{
  int c,
      I = 0;

  while (c = *src++)
  {
    dest[I] = I >= n ? c : toupper (c);
    I++;
  }
  dest[I] = '\0';

  return (dest);
}

/**
 * Converts the first >Lowercase< chars in src to lowercase, and stores the
 * result in dest
 */
char *StrLowConv (char *dest, char *src, int n)
{
  int c,
      I = 0;

  while (c = *src++)
  {
    dest[I] = I >= n ? c : tolower (c);
    I++;
  }
  dest[I] = '\0';

  return (dest);
}

/**
 * Replaces character >Replace[0]< by >Replace[1]<
 */
char *StrReplace (char *dest, char *src)
{
  int c,
      I = 0;

  while (c = *src++)
    dest[I++] = c == Replace[0] ? Replace[1] : c;
  dest[I] = '\0';

  return (dest);
}


/**
 * Checks, wether the length of a word is in the range MinLength-MaxLength
 */
#define LengthOk(l) ((l) >= MinLength && (l) <= MaxLength)

/**
 * Does all the funny things, the user wants with the given word
 */
void ProcessWord (char *word, int wlen)
{
  char str[18];
  int  len, I;

  word[wlen < TruncLength ? wlen : TruncLength] = '\0';

  if (LengthOk (wlen))
  {
    if (IncludeOrigin)
      AddWord (word);
    if (Reverse)
      AddWord (StrReverse (&str[9], word));
    if (Replace[0])
      AddWord (StrReplace (str, word));

    for (I = 0; I < LowConvNum; I++)
      AddWord (StrLowConv (str, word, Lowercase[I]));
    for (I = 0; I < UppConvNum; I++)
      AddWord (StrUppConv (str, word, Uppercase[I]));
  }
  if (LengthOk (wlen+1))
  {
    /* only suspend, if word is so small, that suspended char is among the
       first 8 chars */
    if (Suffixes[0] && wlen < 8)
      AddWordsSuffixes (word, wlen);
    if (Prefixes[0])
      AddWordsPrefixes (word, wlen);
  }
  if (NoVowels)
    if ((len = StrKillVowels (str, word)) >= MinLength && len <= MaxLength)
      AddWord (str);
}

/**
 * Checks wether a character can belong to a word, depending on the
 * commandline options -Alpha and -Chars:string
 */
__inline__ int IsOkChr(c)
{
   if (Alpha)
     return (isalpha (c));
   else if (!OnlyChars[0])
     return ((c > 32 && c <= 255) || (c == 32 && !WholeWords));
   else
     return (strchr (OnlyChars, c) != NULL);
}


/**
 * Gets all words from the dictionary or text file, and calls the word
 *  proccessing routine
 */
void Extract (const char *name)
{
  FILE        *hdl;
  int         I = 0,
              c;
  long        flen;
  char        disp = 0;
  static char word[257];

  if (!stricmp (name, ".stdin"))
    hdl = stdin;
  else if ((hdl = fopen (name, "rb")) == NULL)
  {
    fprintf (stderr, "Can't open %s: %s", name, strerror (errno));
    exit (1);
  }

  flen = filelength (fileno (hdl));

  while (!IsOkChr (c) && !feof (hdl)) c = fgetc (hdl);
  ungetc (c, hdl);

  while (!feof (hdl) && Words < MAXWORDS)
  {
    if (disp++ == 0)
    {
      fprintf (stderr, "\rProcessing %-61s...%3i%%", name, (int) (ftell (hdl)*100 / flen));
      fflush (stderr);
    }
    if (I < 256 && IsOkChr (c = fgetc (hdl)))
      word[I++] = c;
    else
    {
      ProcessWord (word, I);
      while (!IsOkChr (c = fgetc (hdl)) && !feof (hdl));
      ungetc (c, hdl);
      I = 0;
    }
  }

  fprintf (stderr, "\rProcessing %-61s...100%%\n", name);

  if (hdl != stdin)
    fclose (hdl);
}


/**
 * Search for all files matching wildcard. Also recurse subdirectories.
 * The files that are found are processed by Extract.
 */
void WildcardRecurse (char *wildcard)
{
  char  wc_drive[MAXDRIVE],
        wc_dir[MAXDIR],
        wc_file[MAXFILE],
        wc_ext[MAXEXT],
        ff_file[MAXFILE],
        ff_ext[MAXEXT],
        newpath[MAXPATH],
        rec_wc[MAXPATH];
  int   wc_dirlen,
        rslt;
  struct ffblk search;

  /* search and extract all the files, specified by wildcard */
  fnsplit (wildcard, wc_drive, wc_dir, wc_file, wc_ext);
  rslt = findfirst (wildcard, &search, FA_ARCH|FA_RDONLY);

  while (!rslt)
  {
    fnsplit (search.ff_name, NULL, NULL, ff_file, ff_ext);
    fnmerge (newpath, wc_drive, wc_dir, ff_file, ff_ext);
    Extract (newpath);

    rslt = findnext (&search);
  }

  /* search all subdirectories, and extract them by recursion */
  fnmerge (rec_wc, wc_drive, wc_dir, "*", "*");
  rslt = findfirst (rec_wc, &search, FA_DIREC);

  while (!rslt)
  {
    if ((search.ff_attrib & FA_DIREC) && search.ff_name[0] != '.')
    {
      wc_dirlen = strlen (wc_dir);
      strcat (wc_dir, search.ff_name);
      fnmerge (newpath, wc_drive, wc_dir, wc_file, wc_ext);
      WildcardRecurse (newpath);
      wc_dir[wc_dirlen] = '\0';
    }
    rslt = findnext (&search);
  }

}


void SubstractDict ()
{
  FILE  *hdl;
  char  disp,
        word[9],
        *lbrk;
  int   c;
  long  flen;

  if ((hdl = fopen (SubName, "rt")) == NULL)
  {
    fprintf (stderr, "\nCan't open %s: %s\n\n", SubName, strerror (errno));
    exit (1);
  }

  flen = filelength (fileno (hdl));

  for (;;)
  {
    if (fgets (word, 9, hdl) == NULL)
      break;
    if ((lbrk = strchr (word, '\n')) == NULL)
      while ((c = fgetc (hdl)) != '\n' && c != EOF);
    else
      *lbrk = '\0';

    KillWord (word);

    if (!disp++)
      fprintf (stderr, "\rSubstracting %-59s...%3i%%",
                       SubName, (int)(ftell (hdl) * 100 / flen));
  }
  fprintf (stderr, "\rSubstracting %-59s...100%%\n", SubName);
  fprintf (stderr, "%li Words eliminated.\n", KilledWords);
  Words -= KilledWords;
}

/**
 * Prints the dictionary's contents to stdout
 */
void PrintDict ()
{
  int  I;
  char str[9],
       word[8];
  FILE *outhdl;

  /*-----------------------------*/
  void RecursePrint (long i)
  {
    if (i == NOINDEX || getDictKilled (i))
      return;
    RecursePrint (getDictLower (i));

    /* read word, and reverse byte order; intel => raw */
    for (*(long long *)word = getDictStr (i), I = 0; I < 8; I++)
      str[I] = word[7-I];

    fputs (str, outhdl);
    fputc ('\n', outhdl);

    RecursePrint (getDictUpper (i));
  }
  /*-----------------------------*/

  if (!Words)
  {
    fputs ("\nNo words in dictionary!\n", stderr);
    return;
  }

  if (!OutName[0])
  {
    outhdl = stdout;
    strcpy (OutName, ".stdout");
  }
  else if ((outhdl = fopen (OutName, "wb")) == NULL)
  {
    fprintf (stderr, "\nCan't create output file \"%s\": %s\n\n",
                     OutName, strerror (errno));
    exit (1);
  }

  fprintf (stderr, "\nPrinting %li words to \"%s\"...\n", Words, OutName);
  str[8] = '\0';
  RecursePrint (0);
}


/**
 * Ctrl+C - function
 */
void Abort (int dummy)
{
  CloseSwap ();
  fputs ("\n\nAborted!\n\n", stderr);
  exit (1);
}

void Help ()
{
  char  *helptext[] =
  {
    "\nASMCDict [[@]file1] [[@]file2] [...] Options",
    "file  : filename, that can contain wildcards",
    "@file : just like file, but will also recurse subdirectries",
    "Options:",
    "\t-Alpha           only read words that consist of A-Z,a-z (by this",
    "\t                 .txt, .doc and similar files can be read word by word)",
    "\t-Chars:string    only read words that consist of chars from \"string\"",
    "\t-Help            show this screen",
    "\t-Include         include original words to dictionary",
    "\t-Lowercase       add lowercase-converted words to dictionary",
    "\t-Lowercase:num   convert first \"num\" chars in words to lowercase",
    "\t-MAxlength:num   only add words to dictionary, if they aren't longer",
    "\t                 than \"num\" characters (default:200)",
    "\t-MInlength:num   only add words to dictionary, if they aren't shorter",
    "\t                 than \"num\" characters (default:2)",
    "\t-Novowels        add vowel-eliminated words to dictionary",
    "\t-Outfile:name    specify an output filename",
    "\t-Prefixes:string prepend characters from \"string\" to words",
    "\t-REVerse         add reversed words to dictionary",
    "\t-REPlace:XY      replace character 'X' by 'Y'",
    "\t-SUBstract:dict  substract words from \"dict\" from current dictionary",
    "\t-SUFfixes:string suspend characters from \"string\" to words",
    "\t-Truncate:num    truncate the orig. words down to \"num\" characters",
    "\t                 before making the modifications on them",
    "\t-Uppercase       add Uppercase converted words (foo->Foo)",
    "\t-Uppercase:num   convert first \"num\" chars in words to uppercase",
    ""
  };
  int   I;

  for (I = 0; helptext[I][0]; I++)
    puts (helptext[I]);
}

/*******
 * MAIN   - processes arguments
 */
int main (int argn, char *argv[])
{
  int   I,
        fromstdin;

  /*-------------------------------------------------------*/
  /* compares the argument n with str, where the argument must be at least
     chrs characters long */
  int ArgCmp (int n, char *str, char chrs)
  {
    int I, c;

    for (I = 1; (c = tolower (argv[n][I])) == tolower (*str++) &&
                c != '\0' && c != ':'; I++);

    if ((c == '\0' || c == ':') && I > chrs)
      return (1);
    return (0);
  }

  void Invalid (int n)
  {
    fprintf (stderr, "Invalid argument: \"%s\"\n", argv[n]);
    Help ();
    exit (1);
  }

  /* reads a string from an argument, such as: "-arg:abcde" */
  char *ReadArgStr (char *str, int n)
  {
    char *col;

    if ((col = strchr (argv[n], ':')) == NULL)
      Invalid (n);

    strcpy (str, col+1);
    return (str);
  }

  /* reads an integer from an argument, such as: "-arg:1234" */
  int ReadArgInt (int n, int min, int max)
  {
    char *col;
    int  v;

    if ((col = strchr (argv[n], ':')) == NULL ||
        !sscanf (col+1, "%i", &v) ||
        v > max || v < min)
      Invalid (n);

    return (v);
  }

  /* reads an optional integer from an argument "-arg:1234" or "-arg" */
  int ReadArgOptInt (int n, int min, int max, int def)
  {
    char *col;
    int  v;

    if ((col = strchr (argv[n], ':')) == NULL)
      v = def;
    else if (!sscanf (col+1, "%i", &v) ||
             v > max || v < min)
      Invalid (n);

    return (v);
  }
  /*-------------------------------------------------------*/

  fputs ("\n\t--------------------------------------------------------------\n", stderr);
  fputs (  "\t    ASMCDict Version "VERSION_STR" Copyright (C) 1998 David Khling    \n", stderr);
  fputs (  "\t      -- dictionary generator/preprocessor for ASMCrack --    \n", stderr);
  fputs (  "\t--------------------------------------------------------------\n", stderr);
  fputs (  "\t ASMCDict comes with ABSOLUTELY NO WARRENTY. It is free       \n", stderr);
  fputs (  "\t Software and may be distributed under the terms of the       \n", stderr);
  fputs (  "\t GNU General Public License; see gpl.txt for details.         \n", stderr);
  fputs (  "\t If you didn't receive this file, write to the Free Software  \n", stderr);
  fputs (  "\t Foundation, Inc., 59 Temple Place - Suite 330, Boston,       \n", stderr);
  fputs (  "\t MA 02111-1307, USA.                                          \n", stderr);
  fputs (  "\t--------------------------------------------------------------\n\n", stderr);

  for (I = 1; I < argn; I++)
    if (argv[I][0] == '-')
    {
      if      (ArgCmp (I, "Alpha",       1))
        Alpha = 1;
      else if (ArgCmp (I, "Chars",       1))
        ReadArgStr (OnlyChars, I);
      else if (ArgCmp (I, "Help",        1))
        Help (), exit (0);
      else if (ArgCmp (I, "Include",     1))
        IncludeOrigin = 1;
      else if (ArgCmp (I, "Lowercase",   1))
        Lowercase[LowConvNum++&7] = ReadArgOptInt (I, 1, 8, 8);
      else if (ArgCmp (I, "MAxlength",   2))
        MaxLength = ReadArgInt (I, 1, 255);
      else if (ArgCmp (I, "MInlength",   2))
        MinLength = ReadArgInt (I, 1, 255);
      else if (ArgCmp (I, "Novowels",    1))
        NoVowels = 1;
      else if (ArgCmp (I, "Outfile",     1))
        ReadArgStr (OutName, I);
      else if (ArgCmp (I, "Prefixes",    1))
        ReadArgStr (Prefixes, I);
      else if (ArgCmp (I, "REPlace",     3))
        { if (strlen (ReadArgStr (Replace, I)) != 2) Invalid (I); }
      else if (ArgCmp (I, "REVerse",     3))
        Reverse = 1;
      else if (ArgCmp (I, "SUFfixes",    3))
        ReadArgStr (Suffixes, I);
      else if (ArgCmp (I, "SUBstract",   3))
        ReadArgStr (SubName, I);
      else if (ArgCmp (I, "Truncat",     1))
        TruncLength = ReadArgInt (I, 1, 8);
      else if (ArgCmp (I, "Uppercase",   1))
        Uppercase[UppConvNum++&7] = ReadArgOptInt (I, 1, 8, 1);
      else if (ArgCmp (I, "Wholewords",  1))
        WholeWords = 1;
      else
        Invalid (I);
    }

  /* check arguments... */
  if (MinLength > MaxLength)
  {
    fputs ("\nError: Minlength > MaxLength\n\n", stderr);
    exit (1);
  }


  CreateSwap ();      /* create the hard disk swapping file */
  signal (SIGINT, Abort);

  for (I = 1, fromstdin = 1; I < argn; I++)
    if (argv[I][0] != '-')
    {
      if (argv[I][0] == '@')
        WildcardRecurse (&argv[I][1]);
      else
        Extract (argv[I]);
      fromstdin = 0;
    }
  if (fromstdin)
    Extract (".stdin");

  if (Words == MAXWORDS)
    fputs ("Warning: Dictionary overflow - words lost\n", stderr);

  if (SubName[0])
    SubstractDict ();

  PrintDict ();         /* print the dictionary to stdout */
  CloseSwap ();         /* remove the hard disk swapping file */

  return (0);
}
