/******
 * Module of AMSCRACK
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 **
 * Contains:
 *   All the functions, that are expected by ASMCRACK.
 *   The crypt functions of this Module use about 8.5MByte look-up tables.
 *   Should run fast on computers with fast RAM.
 * Needs:
 *   ASMCrypt.c, which creates the small look-up tables, that are needed to
 *   create the huge look-up tables
 */

 #include <stdio.h>


void Info ()
{
  puts ("Subversion: C - crypt functions, using 8.5MByte lookup tables.\n");
}


/*** Table for Left Shifts ***/
int LS[] = {
  1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1
};


typedef union {
   unsigned char  b[8];
   unsigned short i[4];
   struct {
      /* long is often faster than a 32-bit bit field */
      long  i0;
      long  i1;
   } b32;
   long long b64;
} C_block;


long long        PC1_rot_PC2[4][65536],
                 CP_FI[4][65536],
                 PC2r_rot_PC2[2][4][65536],
                 S_P_E[4][65536/4];

extern C_block   PC1ROT[8][256];
extern C_block   PC2ROT[2][8][256];
extern C_block   CF6464[8][256];
extern long      SPE[2][8][64];

/**
 * Initializes my HUGE look-up tables
 */

void Init_DES ()
{
  C_block       I;
  int           J;
  void init_des();

  printf ("Initializing HUGE look-up tables 1-3 ...\n");
  init_des ();  /* I was too lazy, to do this on my own... */

  printf ("\r\t[1]"); fflush (stdout);
  for (I.b64 = 0LL; I.b32.i0 < 65536; I.b32.i0++)
  {
    PC1_rot_PC2[0][I.b32.i0]  = PC1ROT[0][I.b[0]].b64 | PC1ROT[1][I.b[1]].b64;
    PC1_rot_PC2[1][I.b32.i0]  = PC1ROT[2][I.b[0]].b64 | PC1ROT[3][I.b[1]].b64;
    PC1_rot_PC2[2][I.b32.i0]  = PC1ROT[4][I.b[0]].b64 | PC1ROT[5][I.b[1]].b64;
    PC1_rot_PC2[3][I.b32.i0]  = PC1ROT[6][I.b[0]].b64 | PC1ROT[7][I.b[1]].b64;

    CP_FI[0][I.b32.i0] = CF6464[0][I.b[0]].b64 | CF6464[1][I.b[1]].b64;
    CP_FI[1][I.b32.i0] = CF6464[2][I.b[0]].b64 | CF6464[3][I.b[1]].b64;
    CP_FI[2][I.b32.i0] = CF6464[4][I.b[0]].b64 | CF6464[5][I.b[1]].b64;
    CP_FI[3][I.b32.i0] = CF6464[6][I.b[0]].b64 | CF6464[7][I.b[1]].b64;
  }

  printf ("\r\t[2]"); fflush (stdout);
  for (J = 0; J < 2; J++)
    for (I.b64 = 0LL; I.b32.i0 < 65536; I.b32.i0++)
    {
      PC2r_rot_PC2[J][0][I.b32.i0] = PC2ROT[J][0][I.b[0]].b64 | PC2ROT[J][1][I.b[1]].b64;
      PC2r_rot_PC2[J][1][I.b32.i0] = PC2ROT[J][2][I.b[0]].b64 | PC2ROT[J][3][I.b[1]].b64;
      PC2r_rot_PC2[J][2][I.b32.i0] = PC2ROT[J][4][I.b[0]].b64 | PC2ROT[J][5][I.b[1]].b64;
      PC2r_rot_PC2[J][3][I.b32.i0] = PC2ROT[J][6][I.b[0]].b64 | PC2ROT[J][7][I.b[1]].b64;
    }

  printf ("\r\t[3]"); fflush (stdout);
  for (J = 0; J < 65536/4; J++)
  {
    S_P_E[0][J] = 0xFFFFFFFFL&(SPE[0][0][J&63] ^ SPE[0][1][J>>8]);
    S_P_E[0][J] |= (long long)(SPE[1][0][J&63] ^ SPE[1][1][J>>8]) << 32;
    S_P_E[1][J] = 0xFFFFFFFFL&(SPE[0][2][J&63] ^ SPE[0][3][J>>8]);
    S_P_E[1][J] |= (long long)(SPE[1][2][J&63] ^ SPE[1][3][J>>8]) << 32;
    S_P_E[2][J] = 0xFFFFFFFFL&(SPE[0][4][J&63] ^ SPE[0][5][J>>8]);
    S_P_E[2][J] |= (long long)(SPE[1][4][J&63] ^ SPE[1][5][J>>8]) << 32;
    S_P_E[3][J] = 0xFFFFFFFFL&(SPE[0][6][J&63] ^ SPE[0][7][J>>8]);
    S_P_E[3][J] |= (long long)(SPE[1][6][J&63] ^ SPE[1][7][J>>8]) << 32;
  }
  printf ("\r\t   \r"); fflush (stdout);

}

/**
 * Creates the key schedule >K[16]< in 8*(6+2) format, from string >key<
 **
 * NOTE: length l of >key<: 1 <= l <= 8
 */

long long K[16];

void Process_DES_Key (const char *key)
{
  register int  I;
  C_block       r;
  unsigned char c;

  for (I = 0, r.b64 = 0LL; I < 8 && (c = *key++) != '\0'; I++)
    r.b64 |= (long long)c << ((I << 3) + 1);

  /* Key schedule */

  r.b64  = PC1_rot_PC2[0][r.i[0]] | PC1_rot_PC2[1][r.i[1]] |
           PC1_rot_PC2[2][r.i[2]] | PC1_rot_PC2[3][r.i[3]];

  K[0] = r.b64&~0x03030303LL;
  for (I = 1; I < 16; I++)
  {
    r = PC2r_rot_PC2[LS[I]-1][0][r.i[0]] | PC2r_rot_PC2[LS[I]-1][1][r.i[1]] |
        PC2r_rot_PC2[LS[I]-1][2][r.i[2]] | PC2r_rot_PC2[LS[I]-1][3][r.i[3]];
    K[I] = r.b64&~0x03030303LL;
  }
}

/**
 * Cipheres the data 0x0000000000000000LL with the variation >salt<
 */

long long DES_Cipher (long salt)
{
  int                I;
  C_block            L, R,
                     k;
  register C_block   B;

  for (I = 0, R.b64 = L.b64 = 0; I < 12; I++)
  {
    #define CRUNCHBLOCK(a,b,n)                                          \
      k.b32.i1 = k.b32.i0 = (b.b32.i1 ^ b.b32.i0) & salt;               \
      B.b64 = k.b64 ^ b.b64 ^ K[n];                                     \
                                                                        \
      a.b64 ^= *(long long *)((unsigned short *)S_P_E[0] + B.i[0]) ^    \
               *(long long *)((unsigned short *)S_P_E[1] + B.i[1]) ^    \
               *(long long *)((unsigned short *)S_P_E[2] + B.i[2]) ^    \
               *(long long *)((unsigned short *)S_P_E[3] + B.i[3]);     \
                                                                        \
      k.b32.i1 = k.b32.i0 = (a.b32.i1 ^ a.b32.i0) & salt;               \
      B.b64 = k.b64 ^ a.b64 ^ K[n+1];                                   \
                                                                        \
      b.b64 ^= *(long long *)((unsigned short *)S_P_E[0] + B.i[0]) ^    \
               *(long long *)((unsigned short *)S_P_E[1] + B.i[1]) ^    \
               *(long long *)((unsigned short *)S_P_E[2] + B.i[2]) ^    \
               *(long long *)((unsigned short *)S_P_E[3] + B.i[3]);

    CRUNCHBLOCK(L, R,  0); CRUNCHBLOCK(L, R,  2);
    CRUNCHBLOCK(L, R,  4); CRUNCHBLOCK(L, R,  6);
    CRUNCHBLOCK(L, R,  8); CRUNCHBLOCK(L, R, 10);
    CRUNCHBLOCK(L, R, 12); CRUNCHBLOCK(L, R, 14);

    CRUNCHBLOCK(R, L,  0); CRUNCHBLOCK(R, L,  2);
    CRUNCHBLOCK(R, L,  4); CRUNCHBLOCK(R, L,  6);
    CRUNCHBLOCK(R, L,  8); CRUNCHBLOCK(R, L, 10);
    CRUNCHBLOCK(R, L, 12); CRUNCHBLOCK(R, L, 14);
  }

  CRUNCHBLOCK(L, R,  0); CRUNCHBLOCK(L, R,  2);
  CRUNCHBLOCK(L, R,  4); CRUNCHBLOCK(L, R,  6);
  CRUNCHBLOCK(L, R,  8); CRUNCHBLOCK(L, R, 10);
  CRUNCHBLOCK(L, R, 12); CRUNCHBLOCK(L, R, 14);

  R.b32.i0 = (R.b32.i0 >> 3 & 0x0f0f0f0fL) | (R.b32.i1 << 1 & 0xf0f0f0f0L);
  R.b32.i1 = (L.b32.i0 >> 3 & 0x0f0f0f0fL) | (L.b32.i1 << 1 & 0xf0f0f0f0L);

  return (CP_FI[0][R.i[0]] | CP_FI[1][R.i[1]] |
          CP_FI[2][R.i[2]] | CP_FI[3][R.i[3]]);
}

long long Crypt (const char *key, long salt)
{
  if (key)
    Process_DES_Key (key);

  return (DES_Cipher (salt));
}
