/******
 * Module of AMSCRACK
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 **
 * Contains:
 *   C - function, that create the Huge look-up tables for the ASM-Module
 *   Crypth.s .
 *   Also contains some of the other functions, that are needed by ASMCRACK,
 *   and that needn't to be programed in Assembler.
 * Needs:
 *   Xit's Crypt.c, it contains the function, that creates the small (72K)
 *   tables, that are needed, to create the huge tables fast.
 */

#include <stdio.h>

void Info ()
{
  puts ("Subversion: ASM - crypt functions, using 6.5MByte lookup tables.\n");
}

typedef union {
   unsigned char  b[8];
   unsigned short i[4];
   struct {
      long  i0;
      long  i1;
   } b32;
   long long b64;
} C_block;


long long        CP_FI[4][65536],
                 PC2r_rot_PC2[2][4][65536],
                 S_P_E[4][65536/4];

extern C_block   PC2ROT[2][8][256];
extern C_block   CF6464[8][256];
extern long      SPE[2][8][64];

/**
 * Initializes my HUGE look-up tables
 */

void Init_DES ()
{
  C_block       I;
  int           J;
  void init_des();

  printf ("Initializing HUGE look-up tables 1-3 ...\n");
  init_des ();  /* I was too lazy, to do this on my own... */

  printf ("\r\t[1]"); fflush (stdout);
  for (I.b64 = 0LL; I.b32.i0 < 65536; I.b32.i0++)
  {
    CP_FI[0][I.b32.i0] = CF6464[0][I.b[0]].b64 | CF6464[1][I.b[1]].b64;
    CP_FI[1][I.b32.i0] = CF6464[2][I.b[0]].b64 | CF6464[3][I.b[1]].b64;
    CP_FI[2][I.b32.i0] = CF6464[4][I.b[0]].b64 | CF6464[5][I.b[1]].b64;
    CP_FI[3][I.b32.i0] = CF6464[6][I.b[0]].b64 | CF6464[7][I.b[1]].b64;
  }

  printf ("\r\t[2]"); fflush (stdout);
  for (J = 0; J < 2; J++)
    for (I.b64 = 0LL; I.b32.i0 < 65536; I.b32.i0++)
    {
      PC2r_rot_PC2[J][0][I.b32.i0] = PC2ROT[J][0][I.b[0]].b64 | PC2ROT[J][1][I.b[1]].b64;
      PC2r_rot_PC2[J][1][I.b32.i0] = PC2ROT[J][2][I.b[0]].b64 | PC2ROT[J][3][I.b[1]].b64;
      PC2r_rot_PC2[J][2][I.b32.i0] = PC2ROT[J][4][I.b[0]].b64 | PC2ROT[J][5][I.b[1]].b64;
      PC2r_rot_PC2[J][3][I.b32.i0] = PC2ROT[J][6][I.b[0]].b64 | PC2ROT[J][7][I.b[1]].b64;
    }

  printf ("\r\t[3]"); fflush (stdout);
  for (J = 0; J < 65536/4; J++)
  {
    S_P_E[0][J] = 0xFFFFFFFFL&(SPE[0][0][J&63] ^ SPE[0][1][J>>8]);
    S_P_E[0][J] |= (long long)(SPE[1][0][J&63] ^ SPE[1][1][J>>8]) << 32;
    S_P_E[1][J] = 0xFFFFFFFFL&(SPE[0][2][J&63] ^ SPE[0][3][J>>8]);
    S_P_E[1][J] |= (long long)(SPE[1][2][J&63] ^ SPE[1][3][J>>8]) << 32;
    S_P_E[2][J] = 0xFFFFFFFFL&(SPE[0][4][J&63] ^ SPE[0][5][J>>8]);
    S_P_E[2][J] |= (long long)(SPE[1][4][J&63] ^ SPE[1][5][J>>8]) << 32;
    S_P_E[3][J] = 0xFFFFFFFFL&(SPE[0][6][J&63] ^ SPE[0][7][J>>8]);
    S_P_E[3][J] |= (long long)(SPE[1][6][J&63] ^ SPE[1][7][J>>8]) << 32;
  }
  printf ("\r\t   \r"); fflush (stdout);

}

