/******
 * Module of AMSCRACK
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 **
 * Contains:
 *   All the functions, that are expected by ASMCRACK.
 *   This Module is for compiling ASMCRACK without any ASM functions.
 * Needs:
 *   asmcrypt.c
 */


//#define _64BIT_SYSTEM

typedef union {
   unsigned char b[8];
   struct {
      /* long is often faster than a 32-bit bit field */
      long  i0;
      long  i1;
   } b32;
   long l[2];
   long long b64;
} C_block;


extern void init_des ();
extern long long crypt (const char *key, long salt);
extern C_block PC1ROT[8][256],
               PC2ROT[2][8][256],
               CF6464[8][256];
extern long    SPE[2][8][64];
#ifdef _64BIT_SYSTEM
C_block        SPE64[8][64];
#endif

void Info ()
{
  puts ("Subversion: C - crypt functions, using 72K lookup tables.\n");
}

void Init_DES ()
{
  #ifdef _64BIT_SYSTEM
  int I, J;
  #endif
  init_des ();

  #ifdef _64BIT_SYSTEM
  for (I = 0; I < 8; I++)
    for (J = 0; J < 64; J++)
      SPE64[I][J].b32.i0 = SPE[0][I][J], SPE64[I][J].b32.i1 = SPE[1][I][J];
  #endif
}

C_block K[16];

long long Process_DES_Key (const char *key)
{
  #define PC2ROTPERM(s,d,rot)                                         \
    d.b64 = PC2ROT[rot][0][s.b[0]].b64 | PC2ROT[rot][1][s.b[1]].b64 | \
            PC2ROT[rot][2][s.b[2]].b64 | PC2ROT[rot][3][s.b[3]].b64 | \
            PC2ROT[rot][4][s.b[4]].b64 | PC2ROT[rot][5][s.b[5]].b64 | \
            PC2ROT[rot][6][s.b[6]].b64 | PC2ROT[rot][7][s.b[7]].b64,  \
    (*kptr++).b64 = d.b64&0xFFFFFFFFFCFCFCFCLL;


  C_block r, l,
          *kptr;
  unsigned char  c;

  r = PC1ROT[0][*key++<<1].b64;
  if (c = *key++<<1)
  {
    r.b64 |= PC1ROT[1][c].b64;
    if (c = *key++<<1)
    {
      r.b64 |= PC1ROT[2][c].b64;
      if (c = *key++<<1)
      {
        r.b64 |= PC1ROT[3][c].b64;
        if (c = *key++<<1)
        {
          r.b64 |= PC1ROT[4][c].b64;
          if (c = *key++<<1)
          {
            r.b64 |= PC1ROT[5][c].b64;
            if (c = *key++<<1)
            {
              r.b64 |= PC1ROT[6][c].b64;
              if (c = *key++<<1)
                r.b64 |= PC1ROT[7][c].b64;
            }
          }
        }
      }
    }
  }

  kptr = K;
  (*kptr++).b64 = r.b64&0xFFFFFFFFFCFCFCFCLL;

  /* Rotates:  1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1 */
  PC2ROTPERM (r, l, 0);
  PC2ROTPERM (l, r, 1);
  PC2ROTPERM (r, l, 1);
  PC2ROTPERM (l, r, 1);
  PC2ROTPERM (r, l, 1);
  PC2ROTPERM (l, r, 1);
  PC2ROTPERM (r, l, 1);
  PC2ROTPERM (l, r, 0);
  PC2ROTPERM (r, l, 1);
  PC2ROTPERM (l, r, 1);
  PC2ROTPERM (r, l, 1);
  PC2ROTPERM (l, r, 1);
  PC2ROTPERM (r, l, 1);
  PC2ROTPERM (l, r, 1);
  PC2ROTPERM (r, l, 0);
}

#ifndef _64BIT_SYSTEM

long long DES_cipher (long salt)
{
  #define CRUNCH_SMALL(a0,a1,b0,b1,kn)                              \
    k = (b0 ^ b1) & salt;                                           \
    B.b32.i0 = k ^ b0 ^ K[kn].b32.i0;                               \
    B.b32.i1 = k ^ b1 ^ K[kn].b32.i1;                               \
    a0 ^= *(long long *)(((char *)SPE[0][0]) + (tmp = B.b[0]));     \
    a1 ^= *(long long *)(((char *)SPE[1][0]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][1]) + (tmp = B.b[1]));     \
    a1 ^= *(long long *)(((char *)SPE[1][1]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][2]) + (tmp = B.b[2]));     \
    a1 ^= *(long long *)(((char *)SPE[1][2]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][3]) + (tmp = B.b[3]));     \
    a1 ^= *(long long *)(((char *)SPE[1][3]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][4]) + (tmp = B.b[4]));     \
    a1 ^= *(long long *)(((char *)SPE[1][4]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][5]) + (tmp = B.b[5]));     \
    a1 ^= *(long long *)(((char *)SPE[1][5]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][6]) + (tmp = B.b[6]));     \
    a1 ^= *(long long *)(((char *)SPE[1][6]) + tmp);                \
    a0 ^= *(long long *)(((char *)SPE[0][7]) + (tmp = B.b[7]));     \
    a1 ^= *(long long *)(((char *)SPE[1][7]) + tmp);

  C_block   B;
  long      k,
            L0 = 0, L1 = 0,
            R0 = 0, R1 = 0;
  int       I,
            tmp;


  I = 12;

  do
  {
    CRUNCH_SMALL (L0, L1, R0, R1,  0);  CRUNCH_SMALL (R0, R1, L0, L1,  1);
    CRUNCH_SMALL (L0, L1, R0, R1,  2);  CRUNCH_SMALL (R0, R1, L0, L1,  3);
    CRUNCH_SMALL (L0, L1, R0, R1,  4);  CRUNCH_SMALL (R0, R1, L0, L1,  5);
    CRUNCH_SMALL (L0, L1, R0, R1,  6);  CRUNCH_SMALL (R0, R1, L0, L1,  7);
    CRUNCH_SMALL (L0, L1, R0, R1,  8);  CRUNCH_SMALL (R0, R1, L0, L1,  9);
    CRUNCH_SMALL (L0, L1, R0, R1, 10);  CRUNCH_SMALL (R0, R1, L0, L1, 11);
    CRUNCH_SMALL (L0, L1, R0, R1, 12);  CRUNCH_SMALL (R0, R1, L0, L1, 13);
    CRUNCH_SMALL (L0, L1, R0, R1, 14);  CRUNCH_SMALL (R0, R1, L0, L1, 15);
  
    CRUNCH_SMALL (R0, R1, L0, L1,  0);  CRUNCH_SMALL (L0, L1, R0, R1,  1);
    CRUNCH_SMALL (R0, R1, L0, L1,  2);  CRUNCH_SMALL (L0, L1, R0, R1,  3);
    CRUNCH_SMALL (R0, R1, L0, L1,  4);  CRUNCH_SMALL (L0, L1, R0, R1,  5);
    CRUNCH_SMALL (R0, R1, L0, L1,  6);  CRUNCH_SMALL (L0, L1, R0, R1,  7);
    CRUNCH_SMALL (R0, R1, L0, L1,  8);  CRUNCH_SMALL (L0, L1, R0, R1,  9);
    CRUNCH_SMALL (R0, R1, L0, L1, 10);  CRUNCH_SMALL (L0, L1, R0, R1, 11);
    CRUNCH_SMALL (R0, R1, L0, L1, 12);  CRUNCH_SMALL (L0, L1, R0, R1, 13);
    CRUNCH_SMALL (R0, R1, L0, L1, 14);  CRUNCH_SMALL (L0, L1, R0, R1, 15);
  }
  while (--I);

  CRUNCH_SMALL (L0, L1, R0, R1,  0);  CRUNCH_SMALL (R0, R1, L0, L1,  1);
  CRUNCH_SMALL (L0, L1, R0, R1,  2);  CRUNCH_SMALL (R0, R1, L0, L1,  3);
  CRUNCH_SMALL (L0, L1, R0, R1,  4);  CRUNCH_SMALL (R0, R1, L0, L1,  5);
  CRUNCH_SMALL (L0, L1, R0, R1,  6);  CRUNCH_SMALL (R0, R1, L0, L1,  7);
  CRUNCH_SMALL (L0, L1, R0, R1,  8);  CRUNCH_SMALL (R0, R1, L0, L1,  9);
  CRUNCH_SMALL (L0, L1, R0, R1, 10);  CRUNCH_SMALL (R0, R1, L0, L1, 11);
  CRUNCH_SMALL (L0, L1, R0, R1, 12);  CRUNCH_SMALL (R0, R1, L0, L1, 13);
  CRUNCH_SMALL (L0, L1, R0, R1, 14);  CRUNCH_SMALL (R0, R1, L0, L1, 15);

  B.b32.i0 = (R0 >> 3 & 0x0f0f0f0fL) | (R1 << 1 & 0xf0f0f0f0L);
  B.b32.i1 = (L0 >> 3 & 0x0f0f0f0fL) | (L1 << 1 & 0xf0f0f0f0L);

  return (CF6464[0][B.b[0]].b64 | CF6464[1][B.b[1]].b64 |
          CF6464[2][B.b[2]].b64 | CF6464[3][B.b[3]].b64 |
          CF6464[4][B.b[4]].b64 | CF6464[5][B.b[5]].b64 |
          CF6464[6][B.b[6]].b64 | CF6464[7][B.b[7]].b64);
}

#else
long long DES_cipher (long salt)
{
  #define CRUNCH_SMALL(_a,_b,kn)                       \
    k = (((long)_b) ^ ((long)(_b>>32))) & salt;        \
    B.b64 = (k|k<<32) ^ _b ^ K[kn].b64;                \
    _a ^= SPE64[0][B.b[0]>>2].b64;                     \
    _a ^= SPE64[1][B.b[1]>>2].b64;                     \
    _a ^= SPE64[2][B.b[2]>>2].b64;                     \
    _a ^= SPE64[3][B.b[3]>>2].b64;                     \
    _a ^= SPE64[4][B.b[4]>>2].b64;                     \
    _a ^= SPE64[5][B.b[5]>>2].b64;                     \
    _a ^= SPE64[6][B.b[6]>>2].b64;                     \
    _a ^= SPE64[7][B.b[7]>>2].b64;                     \

  C_block   B;
  unsigned long long R = 0,
            L = 0,
            k;
  int       I;


  I = 12;

  do
  {
    CRUNCH_SMALL (L, R,  0);  CRUNCH_SMALL (R, L,  1);
    CRUNCH_SMALL (L, R,  2);  CRUNCH_SMALL (R, L,  3);
    CRUNCH_SMALL (L, R,  4);  CRUNCH_SMALL (R, L,  5);
    CRUNCH_SMALL (L, R,  6);  CRUNCH_SMALL (R, L,  7);
    CRUNCH_SMALL (L, R,  8);  CRUNCH_SMALL (R, L,  9);
    CRUNCH_SMALL (L, R, 10);  CRUNCH_SMALL (R, L, 11);
    CRUNCH_SMALL (L, R, 12);  CRUNCH_SMALL (R, L, 13);
    CRUNCH_SMALL (L, R, 14);  CRUNCH_SMALL (R, L, 15);
  
    CRUNCH_SMALL (R, L,  0);  CRUNCH_SMALL (L, R,  1);
    CRUNCH_SMALL (R, L,  2);  CRUNCH_SMALL (L, R,  3);
    CRUNCH_SMALL (R, L,  4);  CRUNCH_SMALL (L, R,  5);
    CRUNCH_SMALL (R, L,  6);  CRUNCH_SMALL (L, R,  7);
    CRUNCH_SMALL (R, L,  8);  CRUNCH_SMALL (L, R,  9);
    CRUNCH_SMALL (R, L, 10);  CRUNCH_SMALL (L, R, 11);
    CRUNCH_SMALL (R, L, 12);  CRUNCH_SMALL (L, R, 13);
    CRUNCH_SMALL (R, L, 14);  CRUNCH_SMALL (L, R, 15);
  }
  while (--I);

  CRUNCH_SMALL (L, R,  0);  CRUNCH_SMALL (R, L,  1);
  CRUNCH_SMALL (L, R,  2);  CRUNCH_SMALL (R, L,  3);
  CRUNCH_SMALL (L, R,  4);  CRUNCH_SMALL (R, L,  5);
  CRUNCH_SMALL (L, R,  6);  CRUNCH_SMALL (R, L,  7);
  CRUNCH_SMALL (L, R,  8);  CRUNCH_SMALL (R, L,  9);
  CRUNCH_SMALL (L, R, 10);  CRUNCH_SMALL (R, L, 11);
  CRUNCH_SMALL (L, R, 12);  CRUNCH_SMALL (R, L, 13);
  CRUNCH_SMALL (L, R, 14);  CRUNCH_SMALL (R, L, 15);

  B.b32.i0 = (((R >> 3) & 0x0f0f0f0fL) | ((R >> 31) & 0xf0f0f0f0L));
  B.b32.i1 = (((L >> 3) & 0x0f0f0f0fL) | ((L >> 31) & 0xf0f0f0f0L));

  return (CF6464[0][B.b[0]].b64 | CF6464[1][B.b[1]].b64 |
          CF6464[2][B.b[2]].b64 | CF6464[3][B.b[3]].b64 |
          CF6464[4][B.b[4]].b64 | CF6464[5][B.b[5]].b64 |
          CF6464[6][B.b[6]].b64 | CF6464[7][B.b[7]].b64);
}
#endif

long long Crypt (const char *key, long salt)
{
  if (key)
    Process_DES_Key (key);

  return (DES_cipher (salt));
}
