/******
 * MAIN Module of AMSCRACK
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 **
 * Contains:
 *   All the functions, that are used to load accounts, and to crack.
 *   - It's ASMCRACK itsself - only the crypt functions are somewhere else.
 * Requires:
 *   ASMcrypt.c, and the some of the following modules:
 *      + Crypts.s  -  ASM-Crypt functions, 72K look-up tables (works
 *                     best at my K5 166MHz)
 *      + Crypth_s.c + Crypth.s
 *                  -  (almost) ASM-Crypt functions, 6.5Mbyte look-up tables
 *      + Cryptm_s.c + Cryptm.s
 *                  - (almost) ASM-Crypt functions, 580KByte look-up tables
 **
 * NOTE:
 *   You will have to load down the DJGPP - C - compiler, (http://www.delorie.com)
 *   because it uses some specialities of DJGPP - C:
 *     - 64bit integers (long long)
 *     - nested functions
 *   The ASM-Modules are written, using the AT&T - ASM syntax. (looks quite
 *   strange, isn't it) an ASM - compiler for this is included with DJGPP - C.
 */


/************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <signal.h>
#include <keys.h>
#include <pc.h>
#include <fnmatch.h>
/************************************************************************/


/************************************************************************/

/* number of this version */
#define VERSION_STR "2.56"

/* default values */
#define DEFAULT_MinLength       2
#define DEFAULT_MaxLength       8
#define DEFAULT_SmartForceAcc   20
#define DEFAULT_BFChars         "abcdefghijklmnopqrstuvwxyz"
#define DEFAULT_CrackMode       WORDS

#define CRACKED_FILE "asmcrack.got"  /* file for saving all cracked pwds */
#define RESTORE_FILE "asmcrack.rst"  /* file for saving restore informations */

#define CHECKINT     250    /* Check every 250th crypt, wether key was pressed,
                               or wether it is time to save restore infos */
#define MAXSYLLABLES 50000  /* Maximum number of syllables for syllable word
                               creation statistics */
#define MAXSYLLLEN   5      /* maximum length of syllables */

#define MAXGECO      5      /* Maximum number of words in geco field */

#define MAXDISPUSERS 32768  /* Maximum number of users that can be displayed
                               (e.g.: by -cracked) */

/************************************************************************/


/************************************************************************/

/* import from the crypt module (crypts.s/crypth_s.c ... ) */
void           Info ();
void           Init_DES ();
long long      Crypt(const char *, long);
/* import from asmcrypt.c: */
long long      DES_Str2Num (const char *str);
int            ProcessSalt (const char *salt);

/************************************************************************/


/************************************************************************/

typedef struct ACCOUNT
{
  char          *Login,
                *Geco[MAXGECO+1],
                Pwd[14];
  int           SaltNum;
  long long     PwdNum;

  struct ACCOUNT        *Next;

} ACCOUNT;

typedef struct CRACKEDPWD
{
  char          *Login,
                Pwd[9];
  clock_t       Time;
  struct CRACKEDPWD     *Next;
} CRACKEDPWD;

typedef struct OLDCRACKED
{
  char         Crypted[14],
               Cracked[9];
  struct OLDCRACKED     *Next;
} OLDCRACKED;

enum EXITCODES
{
  EX_FINISHED = 0,
  EX_COND_ABORTED,
  EX_ABORTED,
  EX_ERROR,
};

/************************************************************************/


/************************************************************************/

FILE            *DictHdl;                /* Handle of dictionary file */
ACCOUNT         *AccountList;            /* Pointer to linked account list */
CRACKEDPWD      *CrackedList = NULL;     /* Pointer to cracked pwds from this session */
OLDCRACKED      *OldCrackedList = NULL;  /* Pointer to list with allready cracked pwds */

int             Accounts = 0,            /* Number of accounts loaded */
                Salts,                   /* Number of different salts */
                AlertLength = 1,         /* Length of Pwd-Found - Signal */
                Display = 0,             /* Display status every CHECK_INTth crypt */
                SmartForceAcc = 0xFFF,   /* Smart Force Accuracy Value */
                MinLength = 0xFFF,       /* Minimum length of dictwords */
                MaxLength = 0xFFF,       /* Maximum ~ */
                WordSylls = 0xFFF,       /* Number of syllables of new words */
                Valids = 0,              /* Number of cracked pwds */
                ShowCracked = 0,         /* Only show cracked pwds and exit */
                ShowNonCracked = 0,      /* Show non-cracked pwds and exit */
                CondAbrt = 0,            /* Abort caused by -timeabort:num or -vdabort:num */
                AbortOK = 0,             /* Ctrl+c - Abort OK? */
                UsersInList = 0,         /* Number of entries in DisplayUserList */
                MCMUpdate = 0,           /* Reload Accountlist all 5 minutes? */
                NoRestore = 0;           /* Don't store restore-info when aborted */
                                       /* Multi Computer Mode settings */
long            MCMTotalCycles = 0,      /* Number of cycles, that are divided up */
                MCMThisCycles = 0,       /* Number of cycles, that this computer gets */
                MCMBeginningCycle = 0,   /* Beginning cycle, for this computer */
                MCMComputers = 1,        /* Number of computers */

                AbortTime = -1,          /* Time (minutes) until exit */
                AbortVD   = -1,          /* Abort if v/d is below this */
                StartClocks = 0;         /* Additional clock-tics from rst-file */

long long       Crypts = 0,              /* Number of crypts since prog begin */
                CryptsInt = 0;           /* Number of crypts since last keypress */

char            LastWord[256] = "",      /* Last word/account that was tried */
                PwdName[256] = "",       /* Name of Pwd - File */
                DictName[256] = "",      /* Name of Dictionary - File */
                RstName[256] = RESTORE_FILE, /* Name of restore file */
                PwdListName[256] = "",    /* File to store cracked pwds list to */
                UserList[1024] = "",     /* Include User List */
                XUserList[1024] = "",    /* Exclude User List */
                Mask[256] = "",          /* Mask for Smart/Brute Force words */
                BFChars[256] = "",       /* Chars to use for Brute Force */
                MCMOptionStr[256] = "",  /* String, containing the MCM-options */
                *DisplayUserList[MAXDISPUSERS];
                                         /* List of users that are displayed */

enum            CRACKMODE {              /* Cracking mode*/
                  GECO,
                  WORDS,
                  SYLLABLES,
                  SMART_FORCE,
                  BRUTE_FORCE,
                  TEST,
                  FINISHED,
                  DEFAULT = 0xFFF
                } CrackMode = DEFAULT;
int             DictRequired[] = {       /* Modes that require a dictionary */
                  0,    /* Geco       : NO  */
                  1,    /* Words      : YES */
                  1,    /* Syllables  : YES */
                  1,    /* Smart Force: YES */
                  0,    /* Brute Force: NO  */
                  0     /* Test       : NO  */
                };

/************************************************************************/


/************************************************************************/

/**
 * Allocates memory, and prints "Not enough memory"... error if necessary
 */
void *Xmalloc (long bytes)
{
  void *memptr;

  if ((memptr = malloc (bytes)) != NULL)
    return (memptr);

  fputs ("\nNot enough DPMI memory!\n\n", stderr);
  exit (EX_ERROR);
}

/**
 * Allocates memory for string str, and prints "Not enough memory"... error
 * if necessary
 */
void *Xstrdup (char *str)
{
  void *memptr;

  if ((memptr = strdup (str)) != NULL)
    return (memptr);

  fputs ("\nNot enough DPMI memory!\n\n", stderr);
  exit (EX_ERROR);
}


/**
 * Short beep and flash, to signalize something (e.g. Dictionary pass is
 * complete)  Is only executed, if AlertLength is not 0
 */
void ShortSignal ()
{
  if (!AlertLength)
    return;

  sound (440);
  ScreenVisualBell ();
  nosound ();
}

/**
 * Stores the information, needed to continue cracking later...
 */
void StoreStatus ()
{
  FILE          *resthdl;
  int           oldabrt;
  CRACKEDPWD    *Icra;
  static char   *modestr[] =
  {
    "Geco",
    "Words",
    "Syllables",
    "Smart Force",
    "Brute Force"
  };

  /*--------------------------------------------------------*/
  void AddRstEntryS (char *n, char *v) /* add string entry to .rst - file */
    { fprintf (resthdl, "%-12s: %s\n", n, v); }

  void AddRstEntryI (char *n, int v)   /* add int entry to .rst - file */
    { fprintf (resthdl, "%-12s: %i\n", n, v); }

  void AddRstEntryLI (char *n, long v) /* add long int entry to .rst - file */
    { fprintf (resthdl, "%-12s: %li\n", n, v); }

  void AddRstEntryLL (char *n, long long v) /* add long long entry to .rst - file */
    { fprintf (resthdl, "%-12s: %lli\n", n, v); }
  /*--------------------------------------------------------*/

  if (NoRestore)
    return;

  oldabrt = AbortOK; AbortOK = 0;

  if (CrackMode == FINISHED)
  {
    remove (RstName);
    return;
  }

  if (!LastWord[0])
    return;

  if ((resthdl = fopen (RstName, "wt")) == NULL)
  {
    fprintf (stderr, "Can't save restore-information in \"%s\": %s\nWriting to stdout instead.\n\n",
             RstName, strerror (errno));
    fclose (resthdl);
    resthdl = stdout;
    ShortSignal ();
  }

  /* store setttings */
  AddRstEntryS  ("Pwdfile",      PwdName);
  AddRstEntryS  ("Dictfile",     DictName);
  AddRstEntryS  ("Mode",         modestr[CrackMode]);
  AddRstEntryS  ("Last",         LastWord);
  AddRstEntryI  ("Syllables",    WordSylls);
  AddRstEntryI  ("Maxlength",    MaxLength);
  AddRstEntryI  ("Minlength",    MinLength);
  AddRstEntryI  ("Accuracy",     SmartForceAcc);
  AddRstEntryS  ("Chars",        BFChars);
  AddRstEntryS  ("Mask",         Mask);
  AddRstEntryS  ("Userlist",     UserList);
  AddRstEntryS  ("XUserlist",    XUserList);
  AddRstEntryS  ("Multi",        MCMOptionStr);
  AddRstEntryI  ("MCM Update",   MCMUpdate);
  AddRstEntryLI ("Abort Time",   AbortTime);
  AddRstEntryLI ("Abort v/d",    AbortVD);
  AddRstEntryLI ("Clocks",       clock () + StartClocks);
  AddRstEntryLL ("Crypts",       Crypts);
  AddRstEntryI  ("Valids",       Valids);

  /* store cracked pwds+accounts+time_when_cracked */
  for (Icra = CrackedList; Icra != NULL; Icra = Icra->Next)
  {
    AddRstEntryS ("Cracked Log", Icra->Login);
    AddRstEntryS ("Cracked Pwd", Icra->Pwd);
    AddRstEntryLI ("Cracked Clk", Icra->Time);
  }

  fclose (resthdl);
  AbortOK = oldabrt;
}


/**
 * Is called, when the program exits
 */
void BreakFunc (int i)
{
  if (!AbortOK)
    return;

  nosound ();
  fputs ("\nABORTED!\n", stderr);

  if (!NoRestore)
  {
    fprintf (stderr, "Saving restore-information in \"%s\"...\n", RstName);
    StoreStatus ();
  }
  fclose (DictHdl);

  if (CondAbrt)
  {
    ShortSignal ();
    exit (EX_COND_ABORTED);
  }

  exit (EX_ABORTED);
}

/**
 * Signalizes that a password was found
 */
void Alert ()
{
  register int I, J, K;

  for (J = 0; J < AlertLength; J++)
  {
    for (K = 0; K < 3; K++)
      for (I = 1700; I < 3500; I += 60)
      {
        sound (I);
        delay (10);
      }
    nosound ();
    ScreenVisualBell ();
  }
}


/**
 * Converts clock_t to "hours:minutes:seconds"
 */
char *Clocks2Str (clock_t clks)
{
  static char str[20];
  long        secs;
  secs = clks / CLOCKS_PER_SEC;

  sprintf (str, "%02i:%02i:%02i",
    (int) (secs / 3600), (int) ((secs / 60) % 60), (int) (secs % 60));

  return (str);
}


/**
 * Adds an entry to the cracked pwd list (cracked during the current session)
 */
CRACKEDPWD *AddCracked (char *login, char *pwd, long clocks)
{
  static CRACKEDPWD **cur = &CrackedList; /* ptr to ptr to current pwd */
  CRACKEDPWD        *ths;

  *cur          = Xmalloc (sizeof (CRACKEDPWD));
  (*cur)->Next  = NULL;
  (*cur)->Time  = clocks;
  (*cur)->Login = Xstrdup (login);
  strcpy ((*cur)->Pwd, pwd);
  ths = *cur;
  cur = &(*cur)->Next;

  return (ths);
}

/**
 * Prints the cracked password >cracked<
 */
void PrintCracked (CRACKEDPWD *cracked)
{
  long secs;
  secs = cracked->Time / CLOCKS_PER_SEC;

  printf ("[%s]  %-10s: %s\n",
    Clocks2Str (cracked->Time), cracked->Login, cracked->Pwd);
}


/**
 * Frees the memory, used by the account >acc<
 */
void FreeAccount (ACCOUNT *acc)
{
  int I;

  free (acc->Login);
  for (I = 0; acc->Geco[I] != NULL; I++)
    free (acc->Geco[I]);
  free (acc);
}

/**
 * Frees the memory, used by the account list (to reload it)
 */
void FreeAccountList ()
{
  ACCOUNT *Iacc,
          *del;

  for (Iacc = AccountList; Iacc != NULL;)
  {
    del = Iacc; Iacc = Iacc->Next;
    FreeAccount (del);
  }
}

/**
 * Reload the accounts (if cracked-pwds-file changed)
 * is done automatically all 5 Minutes in MCM Mode, if -MReload
 */
void ReloadAccounts ()
{
  void ProcessPwdFile (char *);

  FreeAccountList ();
  ProcessPwdFile (PwdName);
}


/**
 * Trys the password >pwd< at all loaded accounts
 */
void TryPwd (char *pwd)
{
  register int  I;
  int           oldsalt,        /* salt of the last account */
                key;
  long          vperd = 0;      /* valid (cracked) pwds per day */
  long long     encrypted;      /* result of encryption */
  ACCOUNT       *acc,           /* current account */
                **lacc;         /* last account */
  FILE          *pothdl;        /* handle of cracked-pwds-file */
  CRACKEDPWD    *Icra;          /* counter to seek through cracked-pwds-list */
  static long   time = 0,       /* time since program start */
                cstime = 0,     /* time of last c/s - value calculation */
                rsttime = 0,    /* time of last status - saving */
                cryptspersec = 1000,  /* encryptions / seconds */
                totalclocks;    /* number of total clocks of this session */
  static int    Ichk = 0,       /* counts the key etc. interval */
                Imcm = 0;       /* counts the current MCM-cycle */


  /* if MCM: check whether this computer gets the current cycle */
  if (MCMTotalCycles)
  {
    I = Imcm%MCMTotalCycles - MCMBeginningCycle;
    Imcm++;
    if (I < 0 || I >= MCMThisCycles)
      return;
  }

  for (I = 0, acc = AccountList, lacc = &AccountList;
       I < Accounts; I++, lacc = &(acc->Next), acc = acc->Next)
  {
    if (!I)
      encrypted = Crypt (pwd, acc->SaltNum);
    else if (acc->SaltNum != oldsalt)
      encrypted = Crypt (NULL, acc->SaltNum);

    /* Password correct? */
    if (encrypted == acc->PwdNum)
    {
      /* PASSWORD CRACKED!!! */
      PrintCracked (AddCracked (acc->Login, pwd, clock () + StartClocks));
      StoreStatus ();

      if ((pothdl = fopen (CRACKED_FILE, "at")) == NULL)
      {
        perror ("CAN'T STORE PASSWORD IN \""CRACKED_FILE"\"");
        ShortSignal ();
      }
      else
        fprintf (pothdl, "%s:%-10s (possibly %s)\n", acc->Pwd, pwd, acc->Login);
      fclose (pothdl);

      Alert ();
      Valids++;

      *lacc = acc->Next;         /* Remove current entry */
      FreeAccount (acc);
      if (--Accounts == NULL)
      {
        puts ("All loaded accounts cracked.\n");
        CrackMode = FINISHED;
        StoreStatus ();
        exit (0);
      }
    }

    CryptsInt++;
    Ichk++;
    oldsalt = acc->SaltNum;
  }
  strcpy (LastWord, pwd);

  /* Only check for keypress etc. in a special interval.
     And NEVER check keypresses in Test mode */
  if (Ichk < CHECKINT || CrackMode == TEST)
    return;

  Ichk = 0;
  totalclocks = (time = clock ()) + StartClocks;

  /* Save restore information (all 5 minutes), and set >time< */
  /* and reload Accounts, if -mupdate */
  if (time - rsttime > 300*CLOCKS_PER_SEC)
  {
    rsttime = time;
    StoreStatus ();
    if (MCMUpdate)
      ReloadAccounts ();
  }

  /* Check, whether to abort by -timeabort:num or -vdabort:num */
  if ((AbortTime != -1 && totalclocks/(60*CLOCKS_PER_SEC) >= AbortTime) ||
      (Valids > 0 && (vperd = Valids * CLOCKS_PER_SEC*3600*24 /
      (totalclocks == 0 ? 1 : totalclocks)) < AbortVD))
  {
    CondAbrt = 1; /* to cause special exit - code */
    BreakFunc (0);
  }

  /* get a key */
  for (key = 0; kbhit (); key = getxkey ());

  /* Show status ? */
  if (key == K_Space || Display)
  {
    static char *modestrs[] =
    {
      "Geco",
      "Dict. Word",
      "Syll. Word",
      "Smart Force",
      "Brute Force"
    };

    if ((I = time - cstime) > CLOCKS_PER_SEC)
    {
      cryptspersec = CryptsInt*CLOCKS_PER_SEC / I;
      cstime = time;
      Crypts += CryptsInt;
      CryptsInt = 0;
    }

    printf ("[%s] v:%i v/d:%li c:%lli s:%li c/s:%li  %s:%s\r\n",
      Clocks2Str (totalclocks),
      Valids,
      vperd,
      Crypts + CryptsInt,
      totalclocks / CLOCKS_PER_SEC,
      cryptspersec,
      modestrs[CrackMode],
      pwd);
  }
  /* Show cracked pwds ? */
  else if (key == K_Return)
    for (Icra = CrackedList, I = 1; Icra != NULL; Icra = Icra->Next, I++)
    {
      PrintCracked (Icra);
      if (!(I % (ScreenRows () - 1)))
      {
        printf ("Press any key..."); fflush (stdout);
        getxkey ();
        putchar ('\n');
      }
    }
  else if (key == K_Control_R)
    ReloadAccounts ();
}

/**
 * Trys the password >pwd< at the specified account - this function is only
 * used by the Geco cracking mode
 */
int TryPwdOnOneAccount (ACCOUNT *acc, char *pwd)
{
  register int  I;
  int           key;
  long          vperd = 0;      /* valdid (cracked) pwds per day */
  FILE          *pothdl;        /* handle of cracked-pwds-file */
  CRACKEDPWD    *Icra;          /* counter to seek through cracked-pwds-list */
  static long   time = 0,       /* time since program start */
                cstime = 0,     /* time of last c/s - value calculation */
                rsttime = 0,    /* time of last status - saving */
                cryptspersec = 1000,  /* encryptions / seconds */
                totalclocks;    /* number of total clocks of this session */
  static int    Ichk = 0;       /* counts the key etc. interval */


  if (Crypt (pwd, acc->SaltNum) == acc->PwdNum)
  {
    /* PASSWORD CRACKED!!! */
    PrintCracked (AddCracked (acc->Login, pwd, clock () + StartClocks));
    StoreStatus ();

    if ((pothdl = fopen (CRACKED_FILE, "at")) == NULL)
    {
      perror ("CAN'T STORE PASSWORD IN \""CRACKED_FILE"\"");
      ShortSignal ();
    }
    else
      fprintf (pothdl, "%s:%-10s (possibly %s)\n", acc->Pwd, pwd, acc->Login);
    fclose (pothdl);

    Alert ();
    Valids++;
    return (1);
  }

  CryptsInt++;
  Ichk++;

  if (Ichk < CHECKINT)
    return (0);

  Ichk = 0;
  totalclocks = (time = clock ()) + StartClocks;

  /* Save restore information (all 5 minutes), and set >time< */
  /* and reload Accounts, if -mupdate */
  if (time - rsttime > 300*CLOCKS_PER_SEC)
  {
    rsttime = time;
    StoreStatus ();
  }

  /* Check, whether to abort by -timeabort:num or -vdabort:num */
  if ((AbortTime != -1 && totalclocks/(60*CLOCKS_PER_SEC) >= AbortTime) ||
      (Valids > 0 && (vperd = Valids * CLOCKS_PER_SEC*3600*24 /
      (totalclocks == 0 ? 1 : totalclocks)) < AbortVD))
  {
    CondAbrt = 1; /* to cause special exit - code */
    BreakFunc (0);
  }

  /* get a key */
  for (key = 0; kbhit (); key = getxkey ());

  /* Show status ? */
  if (key == K_Space || Display)
  {
    if ((I = time - cstime) > CLOCKS_PER_SEC)
    {
      cryptspersec = CryptsInt*CLOCKS_PER_SEC / I;
      cstime = time;
      Crypts += CryptsInt;
      CryptsInt = 0;
    }

    printf ("[%s] v:%i v/d:%i c:%lli s:%li c/s:%li  Geco(%s):%s\r\n",
      Clocks2Str (totalclocks),
      Valids,
      Valids * CLOCKS_PER_SEC*3600*24 / (totalclocks == 0 ? 1 : totalclocks),
      Crypts + CryptsInt,
      totalclocks / CLOCKS_PER_SEC,
      cryptspersec,
      acc->Login,
      pwd);
  }
  /* Show cracked pwds ? */
  else if (key == K_Return)
    for (Icra = CrackedList, I = 1; Icra != NULL; Icra = Icra->Next, I++)
    {
      PrintCracked (Icra);
      if (!(I % (ScreenRows () - 1)))
      {
        printf ("Press any key..."); fflush (stdout);
        getxkey ();
        putchar ('\n');
      }
    }

  return (0);
}

/**
 * Prints the Cracking... info and switches Ctrl+c-abort on
 */
void Cracking ()
{
  puts ("\nCracking...");
  puts ("  Abort:Ctrl+C  Status:Space  Cracked pwds:Enter  Reload accounts:Ctrl+R\n");
  AbortOK = 1;
}


/**
 * Creates passwords, using the geco - field of the pwd - file
 * The passwords are only tried at the account, the geco field comes from
 */
void GecoCrack ()
{
  ACCOUNT *Account;

  /*----------------------------------------------*/
  /** Concatenate the geco names in any order and with every length and
      different case. Every geco word is only used one time */
  int TryGecoPwds (char *pre, int prelen, int used[])
  {
    register int I, c;
    int          I2;

    if (prelen >= MinLength)
    {
      if (TryPwdOnOneAccount (Account, pre))
        return (1);
    }
    if (prelen == MaxLength)
      return (0);


    for (I2 = 0; Account->Geco[I2] != NULL; I2++)
    {
      if (used[I2])
        continue;
      used[I2] = 1;

      for (I = 0; (c = Account->Geco[I2][I]) && prelen+I < MaxLength;)
      {
        pre[prelen + I++] = c;
        pre[prelen + I] = '\0';
        if (TryGecoPwds (pre, prelen+I, used))
          return (1);
        if (I == 1)
        {
          pre[prelen + I-1] = toupper (c);
          if (TryGecoPwds (pre, prelen+I, used))
            return (1);
          pre[prelen + I-1] = tolower (c);
        }
      }

      used[I2] = 0;
    }

    return (0);
  }

  /** Try geco words truncated to every length, upper and lowercasem,
      with up to 3-digit numbers suspended */
  int TryGecoNumPwds ()
  {
    /*-----------------------------------------*/
    /* recursing function, that suspends >digit< numbers to the word >pre< */
    int TrySuspendNum (char *pre, int prelen, int digits)
    {
      int I;

      if (prelen >= MinLength)
        if (TryPwdOnOneAccount (Account, pre))
          return (1);
      if (prelen >= MaxLength || !digits)
        return (0);

      pre[prelen+1] = '\0'; /* extend the current word by one char */
      for (I = '0'; I <= '9'; I++)
      {
        pre[prelen] = I;
        if (TrySuspendNum (pre, prelen+1, digits-1))
          return (1);
      }
      pre[prelen] = '\0'; /* cut word down to original length */

      return (0);
    }
    /*-----------------------------------------*/

    char geco[9];
    int  I,
         gecoupper,
         gecolen;

    for (I = 0; Account->Geco[I] != NULL; I++)
    {
      /* copy geco word to pwd */
      strncpy (geco, Account->Geco[I], 7);
      geco[7] = '\0';

      /* reduce length of geco till minlength-3 or 1 */
      for (gecolen = strlen (geco); gecolen >= MinLength-3 && gecolen;
           geco[--gecolen] = '\0')
        for (gecoupper = 0; gecoupper < 2; gecoupper++)
        {
          if (gecoupper)
            geco[0] = toupper (geco[0]);
          else
            geco[0] = tolower (geco[0]);

          /* suspend 3digit numbers to pwd and try the resulting word */
          if (TrySuspendNum (geco, gecolen, 3))
            return (1);
        }
    }

    return (0);
  }

  /*----------------------------------------------*/

  char    str[256];
  int     used[MAXGECO],
          rest = 0;

  if (LastWord[0])
  {
    printf ("Restoring last user: \"%s\"...\n", LastWord);
    rest = 1;
  }
  else
    Cracking ();

  for (Account = AccountList; Account != NULL; Account = Account->Next)
  {
    if (rest)
    {
      if (!strcmp (Account->Login, LastWord))
      {
        rest = 0;
        Cracking ();
      }
      continue;
    }

    bzero (used, sizeof (used));
    if (!(TryGecoPwds (str, 0, used) || TryGecoNumPwds ()))
      strcpy (LastWord, Account->Login);
  }
  AbortOK = 0;

  if (rest)
  {
    fputs ("\nCan't restore this user.\n\n", stderr);
    exit (EX_ERROR);
  }
}


/**
 * Trys all passwords in the dictionary on the loaded accounts
 */
void WordCrack ()
{
  char          pwd[9],
                *lbrk;
  register int  I;

  /* Restore dictionary word */
  I = 0;        /* word found - indicator */
  if (LastWord[0])
  {
    printf ("Restoring last word: \"%s\"...\n", LastWord);
    while (fgets (pwd, 9, DictHdl) != NULL)
    {
      if ((lbrk = strchr (pwd, '\n')) != NULL)
        *lbrk = '\0';
      else
        while ((I = fgetc (DictHdl)) != '\n' && I != EOF);
  
      if (!strcmp (LastWord, pwd))
      {
        I = 1;
        break;
      }
    }
    if (!I)
    {
      fputs ("\nCan't restore this word.\n\n", stderr);
      exit (EX_ERROR);
    }
  }

  /* Cracking .... */
  Cracking ();

  while (fgets (pwd, 9, DictHdl) != NULL)
  {
    if ((lbrk = strchr (pwd, '\n')) != NULL)
      *lbrk = '\0';
    else
      while ((I = fgetc (DictHdl)) != '\n' && I != EOF);

    if (pwd[0] && (I = strlen (pwd)) >= MinLength && I <= MaxLength)
      TryPwd (pwd);
  }

  AbortOK = 0;
}


/**
 * Analyses the dictionary, and gaines information about syllables from it.
 * By this it creates words, the most probable first, the least probable
 * last, and trys these words as passwords.
 */
void SyllableCrack ()
{
  struct SYLLABLE
  {
    char      Str[MAXSYLLLEN+1];
    short     Freq;
    unsigned  Len : 7;
    unsigned  ConMustFollow : 1;
  };


  static struct SYLLABLE
    BgSyllList[MAXSYLLABLES],   /* syllables for word begins */
    M1SyllList[MAXSYLLABLES],   /* all syllables, that begin on a consonant */
    M2SyllList[MAXSYLLABLES];   /* all syllables in the mid of a word */

  long  M1SyllNum = 0,          /* number of syllables in M1SyllList */
        M2SyllNum = 0,          /* number of syllables in M2SyllList */
        BgSyllNum = 0;          /* number of syllables in BgSylllist */
  int   rest = 0,               /* restore word? */
        BgSyll,                 /* current syllable is at the word's begin */
        ConMustFollow;          /* consonant must follow on current syllable */

  /*------------------------------------------------------------*/
  int isvowel (char c)        /* returns whether c is a vowel */
  {
    static char vowels[] = "aeiou";
    return (strchr (vowels, c) != NULL);
  }

  /** Read one syllable from the dictionary **/
  char *ReadSyllable ()
  {
    static char syl[9],         /* static array for storing the syllable to */
                word[256];      /* current word */
    static int  wcpos = 0xFFF;  /* number of current character in cur. word*/

    int         state = 0,      /* state of the syllable reading */
                c,              /* current character */
                scpos = 0;      /* number of current character in cur. syllable */
    char        *linebrk;


    if (wcpos == 0xFFF)           /* read new word */
    {
      if (fgets (word, 256, DictHdl) == NULL)
        return (NULL);
      if (linebrk = strchr (word, '\n'))
        *linebrk = '\0';
      else
        while ((c = fgetc (DictHdl)) != '\n' && c != EOF);
      wcpos = 0;
      BgSyll = 1;
    }
    else
      BgSyll = 0;

    while (scpos < 8 && state != 5)
    {
      if (!(c = word[wcpos++]))       /* this word completed */
      {
        wcpos = 0xFFF;
        break;
      }
      switch (state)
      {
        case 0: if (isvowel (c))
                  state = 1;
                break;
        case 1: if (isvowel (c)) state = 2;
                else state = 3;
                break;
        case 2: if (isvowel (c))  /* more than 2 vowels */
                {
                  state = 5;
                  wcpos--;
                }
                else state = 3;
                break;
        case 3: if (isvowel (c)) /* [consonants] [vowel] */
                {
                  wcpos -= 2;           /* unget vowel and last consonant */
                  scpos--;
                  state = 5;
                }
                break;
      }

      if (state != 5)
        syl[scpos++] = c;
    }
    ConMustFollow = 1;
    syl[scpos] = '\0';
    if (isvowel (syl[scpos-1]) && (scpos >= 2 && isvowel (syl[scpos-2])))
      ConMustFollow = 0;
    return (syl);
  }

  /** Add one syllable to syllable-list **/
  void AddSyllable (const char *syl, struct SYLLABLE list[], long *num)
  {
    long upper, lower, mid;
    int  r;

    for (upper = *num, lower = 0;;)
    {
      if (upper == lower)
      {
        if (upper < *num)
        {
          /* syllable allready exists - increase frequency count */
          if (!strcmp (list[upper].Str, syl))
          {
            list[upper].Freq++;
            return;
          }

          if (*num >= MAXSYLLABLES)
            return;
          memmove (&list[upper+1], &list[upper],
                   (*num-upper) * sizeof (struct SYLLABLE));
        }

        /* create new syllable entry */
        if (*num >= MAXSYLLABLES)
          return;
        strcpy (list[upper].Str, syl);
        list[upper].Len = strlen (syl);
        list[upper].Freq = 1;
        list[upper].ConMustFollow = ConMustFollow;
        *num += 1;
        return;
      }
      mid = (upper+lower) >> 1;
      if ((r = strcmp (syl, list[mid].Str)) == 0)
      {
        list[mid].Freq++;
        return;
      }
      else if (r > 0)
        lower = mid+1;
      else if (r < 0)
        upper = mid;
    }
  }

  /** Recursing syllable-word creation, using SyllList **/
  void CreateSyllWords (char *pre, int prelen, int prosum,
         struct SYLLABLE sylllist[], long syllnum, int wordsylls)
  {
    int I, newlen;

    if (rest)
      if (strncmp (pre, LastWord, prelen))
        return;

    if (wordsylls == WordSylls)
    {
      if (prelen < MinLength)
        return;
      if (rest)
      {
        if (!strcmp (pre, LastWord))
        {
          rest = 0;
          Cracking ();
        }
      }
      else
        TryPwd (pre);

      return;
    }

    for (I = WordSylls-1 == wordsylls ? prosum-1 : 0;
         I <= prosum-1 && I < syllnum && I >= 0; I++)
    {
      if ((newlen = prelen+sylllist[I].Len) > MaxLength)
        continue;

      strcpy (&pre[prelen], sylllist[I].Str);
      if (sylllist[I].ConMustFollow)
        CreateSyllWords (pre, newlen, prosum-I-1, M1SyllList, M1SyllNum, wordsylls+1);
      else
        CreateSyllWords (pre, newlen, prosum-I-1, M2SyllList, M2SyllNum, wordsylls+1);
    }
    pre[prelen] = '\0';
  }

  /* Function for sorting the syllable-list */
  int SyllCmp (const void *e1, const void *e2)
  { return (((struct SYLLABLE *)e1)->Freq > ((struct SYLLABLE *)e2)->Freq ? -1 : 1); }

  /*------------------------------------------------------------*/

  long  flen;
  int   I;
  char  *ptrsyl,
        SyllWord[9] = "",
        disp = 0;

  flen = filelength (fileno (DictHdl));

  /* extract syllables from dictioanry */
  do {
    if (ptrsyl = ReadSyllable ())
      if ((I = strlen (ptrsyl)) <= MAXSYLLLEN && I >= 1)
      {
        if (BgSyll)
          AddSyllable (ptrsyl, BgSyllList, &BgSyllNum);
        else
        {
          AddSyllable (ptrsyl, M2SyllList, &M2SyllNum);

          if (!isvowel (ptrsyl[0]))
            AddSyllable (ptrsyl, M1SyllList, &M1SyllNum);
        }
      }

    if (!disp++)
    {
      printf ("\rCreating syllable statistics... %i%%", ftell (DictHdl)*100 / flen);
      fflush (stdout);
    }
  } while (ptrsyl);

  puts ("\rCreating syllable statistics... 100%");

  if (M2SyllNum >= MAXSYLLABLES || BgSyllNum >= MAXSYLLABLES)
    fputs ("Warning: Syllable overflow.\n", stderr);

  printf ("\rSorting syllables... 0%%"); fflush (stdout);
  qsort (M1SyllList, M1SyllNum, sizeof (struct SYLLABLE), SyllCmp);
  printf ("\rSorting syllables... 20%%"); fflush (stdout);
  qsort (M2SyllList, M2SyllNum, sizeof (struct SYLLABLE), SyllCmp);
  printf ("\rSorting syllables... 50%%"); fflush (stdout);
  qsort (BgSyllList, BgSyllNum, sizeof (struct SYLLABLE), SyllCmp);
  puts ("\rSorting syllables... 100%");

  if (LastWord[0])
  {
    rest = 1;
    printf ("Restoring last word: \"%s\"...\n", LastWord);
  }
  else
    Cracking ();

  for (I = WordSylls; I < M2SyllNum*WordSylls; I++)
    CreateSyllWords (SyllWord, 0, I, BgSyllList, BgSyllNum, 0);

  AbortOK = 0;

  if (rest)
  {
    fputs ("\nCan't restore this word.\n\n", stderr);
    exit (EX_ERROR);
  }
}


/**
 * Analyses the dictionary and creates statistic tables. By this it
 * calculate words, the most probable first, the least probable last, and
 * trys these words as passwords.
 */
void SmartForceCrack ()
{
  unsigned char str[256],
                *lbrk;
  register int  c,
                I;
  int           J, K,
                rest = 0;
  static long   bgchfrq[256],        /* frequency of beginning characters */
                cbchfrq[256][256];   /* frequency of character combinations */
  static unsigned char  bgchars[256],        /* beginning chars - sorted max->min */
                        cbchars[256][256];   /* following chars - sorted max->min */

  /*-------------------------------------------------*/
  /** recursing Smart Force word generation */
  void TrySmartForcePwds (unsigned char *pre, int prelen, int prosum)
  {
    register int I, c;

    if (rest)
      if (strncmp (pre, LastWord, prelen))
        return;

    if (prelen > MaxLength)
      return;

    if (!prosum)
    {
      if (prelen < MinLength)
        return;

      if (rest)
      {
        if (!strcmp (pre, LastWord))
        {
          rest = 0;
          Cracking ();
        }
      }
      else
        TryPwd (pre);

      return;
    }


    pre[prelen+1] = '\0';    /* make current word one char longer */
    c = pre[prelen-1]&0xFF;

    if (Mask[0] && Mask[prelen] != '_') /* cur. char is masked */
      for (I = prelen == MaxLength-1 ? prosum-1 : 0;
           I < prosum && I < SmartForceAcc; I++)
      {
        if (!cbchfrq[c][pre[prelen] = cbchars[c][I]&0xFF])
          break;
        if (pre[prelen] == Mask[prelen])
          TrySmartForcePwds (pre, prelen+1, prosum-I-1);
      }
    else   /* suspend the most propable chars... */
      for (I = prelen == MaxLength-1 ? prosum-1 : 0;
           I < prosum && I < SmartForceAcc; I++)
      {
        if (!cbchfrq[c][pre[prelen] = cbchars[c][I]&0xFF])
          break;
        TrySmartForcePwds (pre, prelen+1, prosum-I-1);
      }

    pre[prelen] = '\0';
  }
  /*-------------------------------------------------*/

  bzero (cbchfrq, sizeof (cbchfrq));
  bzero (bgchfrq, sizeof (bgchfrq));

  puts ("Creating Smart Force statistics...");
  while (fgets (str, 256, DictHdl) != NULL)
  {
    if ((lbrk = strchr (str, '\n')) != NULL)
      *lbrk = '\0';
    else
      while ((c = fgetc (DictHdl)) != '\n' && c != EOF);

    if (!str[0])
      continue;

    bgchfrq[c = str[0]&0xFF]++;

    for (I = 1; I < 256 && str[I] != '\0'; c = str[I++]&0xFF) /* else c can get negative values. Type conversion didn't fixed it. (Bug in DJGPP - C ?) */
      cbchfrq[c][str[I]]++;
  }

  /* Sort bgchars by bgchfrq */
  for (I = 0; I < 256; I++)
    bgchars[I] = I;
  for (J = 0; J < 255; J++)
    for (I = J; I < 256; I++)
      if (bgchfrq[bgchars[I]] > bgchfrq[bgchars[J]])
      {
        c = bgchars[I];
        bgchars[I] = bgchars[J];
        bgchars[J] = c;
      }

  /* Sort cbchars by bgchfrq */
  for (J = 0; J < 256; J++)
    for (I = 0; I < 256; I++)
      cbchars[J][I] = I;

  for (K = 0; K < 256; K++)
    for (J = 0; J < 255; J++)
      for (I = J; I < 256; I++)
        if (cbchfrq[K][cbchars[K][I]] > cbchfrq[K][cbchars[K][J]])
        {
          c = cbchars[K][I];
          cbchars[K][I] = cbchars[K][J];
          cbchars[K][J] = c;
        }

  if (LastWord[0])
  {
    rest = 1;
    printf ("Restoring last word: \"%s\"...\n", LastWord);
  }
  else
    Cracking ();

  /**
   * Every Smart Foce word is generated, using a sum, that represents the
   * sum of the indixes of the characters in the frequency-ordered character
   * combination table. A lower propability sum (prosum) means that the
   * character combinations in the word where more frequently in the
   * dictionary.
   */
  if (Mask[0] && Mask[0] != '_')
  {
    for (I = MinLength-1, str[1] = '\0'; I < SmartForceAcc*MaxLength; I++)
      for (J = 0; bgchfrq[str[0] = bgchars[J]&0xFF]; J++)
        if (str[0] == Mask[0])
          TrySmartForcePwds (str, 1, I);
  }
  else
    for (I = MinLength-1, str[1] = '\0'; I < SmartForceAcc*MaxLength; I++)
      for (J = 0; bgchfrq[str[0] = bgchars[J]&0xFF]; J++)
        TrySmartForcePwds (str, 1, I);

  AbortOK = 0;

  if (rest)
  {
    fputs ("\nCan't restore this word.\n\n", stderr);
    exit (EX_ERROR);
  }
}


/**
 * Cracks, using Brute Force, with the given characters from BFChars
 */
void BruteForceCrack ()
{
  int CharNum,
      rest = 0,
      ThisLength;       /* current length of Brute Force words */

  /*-----------------------------------------------------*/
  void TryBruteForcePwds (char *pre, int prelen)
  {
    register int I;

    if (rest && prelen)
      if (strncmp (pre, LastWord, prelen))
        return;

    if (prelen == ThisLength)
    {
      if (rest)
      {
        if (!strcmp (pre, LastWord))
        {
          rest = 0;
          Cracking ();
        }
      }
      else
        TryPwd (pre);

      return;
    }

    pre[prelen+1] = '\0';               /* Add one character to pwd-string */

    if (Mask[0] && Mask[prelen] != '_')
    {                   /* current char = char from mask */
      pre[prelen] = Mask[prelen];
      TryBruteForcePwds (pre, prelen+1);
    }
    else                /* suspend all chars to pwd-string */
      for (I = 0; I < CharNum; I++)
      {
        pre[prelen] = BFChars[I];
        TryBruteForcePwds (pre, prelen+1);
      }

    pre[prelen] = '\0';               /* Cut pwd-string down to orignial length */

  }
  /*-----------------------------------------------------*/

  if (LastWord[0])
  {
    rest = 1;
    printf ("Restoring last word: \"%s\"...\n", LastWord);
  }
  else
    Cracking ();

  CharNum = strlen (BFChars);

  for (ThisLength = MinLength; ThisLength <= MaxLength; ThisLength++)
    TryBruteForcePwds ("", 0);

  AbortOK = 0;

  if (rest)
  {
    fputs ("\nCan't restore this word.\n\n", stderr);
    exit (EX_ERROR);
  }
}

/**
 * Does a speed - test, using random words
 */
void SpeedTestCrack ()
{
  register long I;
  uclock_t time;
  char     pwd[9],
           *text[] =
  {
    "\nNOTE:\tThe speed value, that is calculated now, depends on the number of",
    "\tloaded accounts and the computer's background activities.",
    "\tDon't press any key and never run this under Windows!\n",
    ""
  };

  for (I = 0; text[I][0] != '\0'; I++)
    puts (text[I]);

  puts ("Encrypting dummy pwds to calculate the speed value...");

  /* Some encryptions, to put the often used data into the CPU cache */
  for (I = 0; I < 300000*MCMComputers / Accounts; I++)
  {
    pwd[0] = random ()%26 + 'a'; pwd[1] = random ()%26 + 'a';
    pwd[2] = random ()%26 + 'a'; pwd[3] = random ()%26 + 'a';
    pwd[4] = random ()%26 + 'a'; pwd[5] = random ()%26 + 'a';
    TryPwd (pwd);
  }

  time = uclock ();
  pwd[6] = '\0';
  for (I = 0; I < 300000*MCMComputers / Accounts; I++)
  {
    pwd[0] = random ()%26 + 'a'; pwd[1] = random ()%26 + 'a';
    pwd[2] = random ()%26 + 'a'; pwd[3] = random ()%26 + 'a';
    pwd[4] = random ()%26 + 'a'; pwd[5] = random ()%26 + 'a';

    TryPwd (pwd);
  }
  time = uclock () - time;

  printf ("Speed Value: %li c/s\n", 300000LL*UCLOCKS_PER_SEC*MCMComputers / time);
  if (MCMTotalCycles)
    puts ("(this value represents the work of all MCM computers together)");
}


/**
 * Checks, weither the string >str< is in the list >list<, where strings
 * are listed as str,str2,str3,...
 */
char *InList (const char *str, char *list)
{
  char *elm;
  int  slen;
  char estr[256];

  slen = strlen (str);
  estr[slen] = '\0';

  for (elm = list; elm-1 != NULL; elm = strchr (elm, ',')+1)
  {
     strncpy (estr, elm, slen);
     if ((elm[slen] == '\0' || elm[slen] == ',') &&
         !fnmatch (estr, str, 0))
       return (elm);
  }

  return (NULL);
}


/**
 * Reads the cracked-file >cfhdl<, and stores all the crypted pwds in
 * >OldCrackedList<
 */
void ReadCrackedFile (FILE *cfhdl)
{
  OLDCRACKED **cr;
  char       crypted[14],
             cracked[9];
  int        c;

  for (cr = &OldCrackedList; ; cr = &((*cr)->Next))
  {
    if (fgets (crypted, 14, cfhdl) == NULL)
      break;
    if (fgetc (cfhdl) != ':' || fgets (cracked, 9, cfhdl) == NULL)
    {
      fputs ("\nInvalid cracked-pwds-file \""CRACKED_FILE"\"\n\n", stderr);
      exit (EX_ERROR);
    }

    *cr = Xmalloc (sizeof (OLDCRACKED));
    strcpy ((*cr)->Crypted, crypted);
    strcpy ((*cr)->Cracked, cracked);
    (*cr)->Next = NULL;

    while ((c = fgetc (cfhdl)) != '\n')
      if (c == EOF)
        return;
  }
}

/**
 * Checks whether the encrypted pwd >pwd< is in the cracked-pwds-list
 */
char *IsAllreadyCracked (char *str)
{
  OLDCRACKED *I;
  for (I = OldCrackedList; I != NULL; I = I->Next)
    if (!strncmp (I->Crypted, str, 13))
      return (I->Cracked);

  return (NULL);
}

/**
 * Stores a list of all cracked pwds to >name< and exits
 */
void StorePwdList (char *name)
{
  FILE          *stohdl,        /* handle of file to store the pwd-list to */
                *cfhdl;         /* handle of cracked-pwds-file */
  OLDCRACKED    *I;

  if ((cfhdl = fopen (CRACKED_FILE, "rt")) == NULL)
  {
    perror ("\nCan't read cracked-pwds-file \""CRACKED_FILE"\"");
    exit (EX_ERROR);
  }
  puts ("Reading \""CRACKED_FILE"\"...");
  ReadCrackedFile (cfhdl);
  if ((stohdl = fopen (name, "wb")) == NULL)
  {
    fprintf (stderr, "\nCan't create pwd-list-file \"%s\": %s\n\n",
             name, strerror (errno));
    exit (EX_ERROR);
  }

  printf ("Storing cracked-pwds-list to \"%s\"...\n", name);
  for (I = OldCrackedList; I != NULL; I = I->Next)
  {
    fputs (I->Cracked, stohdl);
    fputc ('\n', stohdl);
  }

  exit (EX_FINISHED);
}

/**
 * Frees the memory, used by the old-cracked-pwds - list
 */
void FreeCrackedFile ()
{
  OLDCRACKED *kill,
             *I;

  for (I = OldCrackedList; I != NULL;)
  {
    kill = I;
    I = I->Next;
    xfree (kill);
  }
}

/**
 * Adds the user to the list with the users that should be displayed
 * (e.g. for the "-cracked" option)
 */
void Add2DisplayUserList (char *login, char *cracked, char *gecoetc)
{
  static char str[256];
  sprintf (str, "%-10s:%-8s:%s", login, cracked, gecoetc);
  DisplayUserList[UsersInList++] = Xstrdup (str);
}

/**
 * Displays the users in >DisplayUserList< in alphabetical order
 */
void DisplayUsersInList ()
{
  /*------------------------------------*/
  int qsort_cmp (const void *e1, const void *e2)
  {
    return (strcmp (*(char **)e1, *(char **)e2));
  }
  /*------------------------------------*/

  int I;

  qsort (DisplayUserList, UsersInList, sizeof (char *), qsort_cmp);

  putchar ('\n');
  for (I = 0; I < UsersInList; I++)
    puts (DisplayUserList[I]);
}

/**
 * Reads the passwordfile >pwdhdl< into the global linked list >Accountlist<,
 * and sets the global var >Accounts< to the number of accounts loaded.
 * It won't load all those old cracked passwords, that are in OldCrackedList.
 * If ShowCracked or ShowNonCracked, the corresponding logins and pwds are
 * added to DisplayUserList
 */
void ReadPwdFile (FILE *pwdhdl)
{
  register int  I, c;
  int           crackedacc = 0,         /* Number of cracked accounts */
                noncrackedacc = 0,      /* Number of non-cracked accounts */
                specacc    = 0,         /* Number of specified accounts */
                inactive,               /* An account is (propably) inactive */
                I2;
  static char   str[256],
                gecoetc[256];           /* geco field, user id, home dir etc. */
  char          *cracked,               /* cracked pwd */
                *sptr;
  ACCOUNT       *acc,                   /* Current account */
                *lacc = NULL;           /* Last account */


  /*--------------------------------------------*/
  void Invalid ()
  {
    fputs ("\nInvalid password file\n\n", stderr);
    exit (EX_ERROR);
  }
  /*--------------------------------------------*/

  for (Accounts = 0, acc = AccountList = Xmalloc (sizeof (ACCOUNT));;)
  {
    acc->Next = NULL;

   /**** read login ****/
    for (I = 0, c = fgetc (pwdhdl);
         I < 256 && c != ':' && c != EOF; I++, c = fgetc (pwdhdl))
      str[I] = c;

    if (I == 256)
      Invalid ();

    if (c == EOF)                       /* Pwdfile ended correctly */
    {
      if (lacc == NULL)
        break;
      lacc->Next = NULL;                /* Destroy current entry */
      xfree (acc);
      break;
    }

    str[I++] = '\0';
    acc->Login = Xstrdup (str);

   /**** read encrypted password ****/
    for (I = 0, c = fgetc (pwdhdl);
         I < 14 && c != ':' && c != ',' && c != EOF; I++, c = fgetc (pwdhdl))
      str[I] = c;
    if (c == ',')
      while ((c = fgetc (pwdhdl)) != ':' && c != EOF)
    if (c == EOF)
      Invalid ();
    inactive = I != 13;

    str[I] = '\0';
    strcpy (acc->Pwd, str);
    acc->SaltNum = ProcessSalt (acc->Pwd);
    acc->PwdNum = DES_Str2Num (acc->Pwd+2);

   /**** read user id, geco field, home dir etc. ****/
    for (I = 0; (c = fgetc (pwdhdl)) != '\n' && c != EOF; I++)
      gecoetc[I] = c;
    gecoetc[I] = '\0';
    if (c == EOF)                       /* Pwdfile ended correctly */
    {
      if (lacc == NULL)
        break;
      lacc->Next = NULL;                /* Destroy current entry */
      free (acc->Login);
      free (acc);
      break;
    }

    /* Inactive account?  User not included? User excluded ? */
    if ((UserList[0] && !InList (acc->Login, UserList)) ||
        (XUserList[0] && InList (acc->Login, XUserList)) ||
        inactive)
    {
      free (acc->Login);
      continue;
    }
    specacc++; /* number of specified accounts ++ */

    /* user is allready cracked ? */
    if (cracked = IsAllreadyCracked (acc->Pwd))
    {
      if (ShowCracked)
        Add2DisplayUserList (acc->Login, cracked, gecoetc);
      crackedacc++;
      free (acc->Login);
      continue;
    }
    else
    {
      if (ShowNonCracked)
        Add2DisplayUserList (acc->Login, "????????", gecoetc);
      noncrackedacc++;
    }

   /**** extract names from geco field ****/

    /* seek to geco field */
    for (I = 0, sptr = gecoetc; I < 2; I++)
      if ((sptr = strchr (sptr, ':')) == NULL) Invalid ();
      else sptr++;

    acc->Geco[0] = Xstrdup (acc->Login); /* the first geco name is the login */
    for (I2 = 1; I2 < MAXGECO;)
    {
      while (!isalpha (c = *sptr++) && c != ':');
      if (c == ':') break;

      I = 0;
      do
        str[I++] = tolower (c);
      while (isalpha (c = *sptr++));
      str[I++] = '\0';

      acc->Geco[I2++] = Xstrdup (str);
      if (c == ':')
        break;
    }
    acc->Geco[I2] = NULL;

   /**** Go to Next Line ****/

    lacc = acc;
    acc = acc->Next = Xmalloc (sizeof (ACCOUNT));
  }

  printf ("    Specified  : %4i\n", specacc);
  if (!specacc)
  {
    if (UserList[0] || XUserList[0])
      fputs ("Warning: No account matches the specifications.\n", stderr);
    else
      fputs ("Warning: No active account in password file.\n", stderr);
    exit (EX_FINISHED);
  }

  printf ("    Cracked    : %4i  (%i%%)\n", crackedacc, crackedacc*100 / specacc);
  printf ("    Remaining  : %4i  (%i%%)\n", noncrackedacc, 100 - crackedacc*100 / specacc);
  if (!noncrackedacc)
  {
    fputs ("Warning: All specified accounts are allready cracked.\n", stderr);
    exit (EX_FINISHED);
  }
  if (ShowCracked || ShowNonCracked)
  {
    DisplayUsersInList ();
    exit (EX_FINISHED);
  }

  Accounts = noncrackedacc;
}


/**
 * Sorts the linked list >AccountList< by salts and sets the
 * global var Salts to the number of different salts
 */
void SortAccountList ()
{
  ACCOUNT *acc,
          *acc2,
          *lacc,
          *xacc,
          *acct;

  for (acc = AccountList, Salts = Accounts;
       acc->Next != NULL && acc->Next->Next != NULL; acc = acc->Next)
    for (acc2 = acc->Next->Next, lacc = acc2->Next;
         acc2 != NULL; lacc = acc2, acc2 = acc2->Next)
      if (acc->SaltNum == acc2->SaltNum)
      {
        xacc = acc->Next;       /* Exchange xacc >acc->Next< with
                                   acc2 >lacc->Next< */
        acct = xacc->Next;
        xacc->Next = acc2->Next;
        acc2->Next = acct;

        acct = acc->Next;
        acc->Next = lacc->Next;
        lacc->Next = acct;

        Salts--;
        break;
      }

  printf ("    Salts      : %4i  (optimisation: %i%%)\n",
          Salts, Accounts*100 / Salts - 100);
}

/**
 * Opens and reads the passwordfile >name<
 */
void ProcessPwdFile (char *name)
{
  FILE          *pwdhdl,
                *cfhdl;

  if ((pwdhdl = fopen (name, "rt")) == NULL)
  {
    perror ("\nCan't open password file");
    exit (EX_ERROR);
  }

  if ((cfhdl = fopen (CRACKED_FILE, "rt")) != NULL)
  {
    puts ("Reading \""CRACKED_FILE"\"...");
    ReadCrackedFile (cfhdl);
  }

  printf ("Loading accounts...\n"); fflush (stdout);
  ReadPwdFile (pwdhdl);
  FreeCrackedFile ();

  if (CrackMode != GECO) /* (Sorting is not necessary in single crack mode) */
    SortAccountList ();

  if (!Accounts)
    exit (EX_ERROR);

  fclose (pwdhdl);
}

/**
 * Opens dictionaryfile >name<
 */
void ProcessDictFile (char *name)
{
  if (!stricmp (name, ".stdin"))
    DictHdl = stdin;
  else if ((DictHdl = fopen (name, "rt")) == NULL)
  {
    perror ("Can't open dictionary file");
    exit (EX_ERROR);
  }
}


/**
 * Opens and reads the restore-file >RstName<
 */
void ProcessRstFile ()
{
  FILE          *resthdl;

  /*----------------------------------------------------*/
  void Invalid ()
  {
    fprintf (stderr, "\nInvalid restore-file \"%s\".\n\n", RstName);
    exit (EX_ERROR);
  }

  char *getRstEntryN ()  /* get the name of the entry */
  {
    static char n[257];
    int         I, c;

    for (I = 0; (c = fgetc (resthdl)) != ':'; I++)
    {
      if (c == EOF || c == '\n' || I > 255 )
        Invalid ();
      n[I] = c;
    }
    n[I] = '\0';

    return (n);
  }

  char *getRstEntryV ()  /* get the value of the current entry */
  {
    static char v[257];
    int         I, c;

    if (fgetc (resthdl) != ' ')
      Invalid ();
    for (I = 0; (c = fgetc (resthdl)) != '\n'; I++)
    {
      if (c == EOF || I > 255)
        Invalid ();
      v[I] = c;
    }
    v[I] = '\0';

    return (v);
  }

  char *getRstEntryS (char *n, char *d)  /* get a string entry */
  {
    if (strncmp (n, getRstEntryN (), strlen (n)))
      Invalid ();
    return (strcpy (d, getRstEntryV ()));
  }

  int getRstEntryI (char *n, int *d)     /* get an int entry */
  {
    if (strncmp (n, getRstEntryN (), strlen (n)))
      Invalid ();
    return (*d = atoi (getRstEntryV ()));
  }

  long getRstEntryLI (char *n, long *d)  /* get a long int entry */
  {
    if (strncmp (n, getRstEntryN (), strlen (n)))
      Invalid ();
    return (*d = atol (getRstEntryV ()));
  }

  long long getRstEntryLL (char *n, long long *d)  /* get a long long entry */
  {
    if (strncmp (n, getRstEntryN (), strlen (n)))
      Invalid ();
    return (*d = strtoll (getRstEntryV (), NULL, 10));
  }
  /*----------------------------------------------------*/

  static char   mode[128];
  int           I;
  char          cralog[256], crapwd[9];
  long          craclk;


  if ((resthdl = fopen (RstName, "rt")) == NULL)
  {
    fprintf (stderr, "\nCan't open restore-file \"%s\": %s\n\n",
                      RstName, strerror (errno));
    exit (EX_ERROR);
  }
  printf ("Reading restore-file \"%s\"...\n", RstName);

  getRstEntryS  ("Pwdfile",      PwdName);
  getRstEntryS  ("Dictfile",     DictName);
  getRstEntryS  ("Mode",         mode);
  getRstEntryS  ("Last",         LastWord);
  getRstEntryI  ("Syllables",    &WordSylls);
  getRstEntryI  ("Maxlength",    &MaxLength);
  getRstEntryI  ("Minlength",    &MinLength);
  getRstEntryI  ("Accuracy",     &SmartForceAcc);
  getRstEntryS  ("Chars",        BFChars);
  getRstEntryS  ("Mask",         Mask);
  getRstEntryS  ("Userlist",     UserList);
  getRstEntryS  ("XUserlist",    XUserList);
  getRstEntryS  ("Multi",        MCMOptionStr);
  getRstEntryI  ("MCM Update",   &MCMUpdate);
  getRstEntryLI ("Abort Time",   &AbortTime);
  getRstEntryLI ("Abort v/d",    &AbortVD);
  getRstEntryLI ("Clocks",       &StartClocks);
  getRstEntryLL ("Crypts",       &Crypts);
  getRstEntryI  ("Valids",       &Valids);

  for (I = 0; I < Valids; I++)
  {
    getRstEntryS  ("Cracked Log", cralog);
    getRstEntryS  ("Cracked Pwd", crapwd);
    getRstEntryLI ("Cracked Clk", &craclk);
    AddCracked (cralog, crapwd, craclk);
  }

  if      (!strcmp (mode, "Geco"))        CrackMode = GECO;
  else if (!strcmp (mode, "Words"))       CrackMode = WORDS;
  else if (!strcmp (mode, "Syllables"))   CrackMode = SYLLABLES;
  else if (!strcmp (mode, "Smart Force")) CrackMode = SMART_FORCE;
  else if (!strcmp (mode, "Brute Force")) CrackMode = BRUTE_FORCE;
  else Invalid ();

  fclose (resthdl);
}


/**
 * Process the Multi Computer Mode options:
 *   - read the cfg - file
 *   - set the global variables
 */
void ProcessMCMOptions (char *opt)
{
  FILE  *CfgHdl;
  int   Line = 0;
  enum  CONFIGERRORS {
    UNEXPECTED_END,
    UNEXPECTED_LBRK,
    SYNTAX_ERROR
  };

  /*------------------------------------------------------*/

  /* print error string and exit */
  void ConfigError (int err)
  {
    fprintf (stderr, "\nError in configuration file, line %i:\n", Line);
    switch (err)
    {
      case UNEXPECTED_END:
        fputs ("\tUnexpected end of file.\n\n", stderr);
        break;
      case UNEXPECTED_LBRK:
        fputs ("\tUnexpected end of line.\n\n", stderr);
        break;
      case SYNTAX_ERROR:
        fputs ("\tSyntax Error.\n\n", stderr);
        break;
    }
    exit (EX_ERROR);
  }

  /* get configuration file entry name: cont */
  int getConfigEntry (char *name, char *cont)
  {
    int c;

    Line++;

    while ((c = fgetc (CfgHdl)) == ' ' || c == '\t' || c == '\n');

    if (c == EOF)
      return (0);

    /* read name of entry */
    do
    {
      if (c == EOF)
        ConfigError (UNEXPECTED_END);
      if (c == '\n')
        ConfigError (UNEXPECTED_LBRK);

      *name++ = c;
    }
    while ((c = fgetc (CfgHdl)) != ':');

    *name = '\0';

    /* seek to the begin of entry's content */
    while ((c = fgetc (CfgHdl)) == ' ' || c == '\t');

    if (c == EOF)
      ConfigError (UNEXPECTED_END);
    else if (c == '\n')
      ConfigError (UNEXPECTED_LBRK);

    /* read content */
    do
      *cont++ = c;
    while ((c = fgetc (CfgHdl)) != EOF && c != '\n');

    *cont = '\0';

    return (1);
  }
  /*------------------------------------------------------*/


  int          I,
               num;             /* number of computer (num parameter) */
  long         v;
  static char  cfgname[256],    /* static for memory reasons */
               entryn[256],
               entryc[256],
               cname[256],
               *col;

  /* Extract cfg-file-name and number of computer from options string */
  strcpy (cfgname, opt);
  if ((col = strchr (cfgname, ',')) == NULL)
  {
    fputs ("\nMissing \"num\" parameter in -multi option\n\n", stderr);
    exit (EX_ERROR);
  }
  *col = '\0';

  if (!sscanf (col+1, "%i", &num))
  {
    fputs ("\nWrong \"num\" parameter in -multi option\n\n", stderr);
    exit (EX_ERROR);
  }

  /* Process the configuration file */
  if ((CfgHdl = fopen (cfgname, "rt")) == NULL)
  {
    fprintf (stderr, "\nCan't open MCM config. file \"%s\": %s\n\n",
                     cfgname, strerror (errno));
    exit (EX_ERROR);
  }
  else
    printf ("Reading MCM config. file \"%s\"...\n", cfgname);

  /* Read the computer-speed-values from config. file */
  for (I = 0; getConfigEntry (entryn, entryc); I++)
  {
    if (!sscanf (entryc, "%li", &v) || v <= 0)
      ConfigError (SYNTAX_ERROR);
    if (I < num)
      MCMBeginningCycle += v;
    else if (I == num)
    {
      MCMThisCycles = v;
      strcpy (cname, entryn);
    }

    MCMTotalCycles += v;
  }

  if (num >= I)
  {
    fputs ("\nNumber of this computer overflows config. file definitions.\n\n", stderr);
    exit (EX_ERROR);
  }

  MCMComputers = I;

  printf ("MCM: This computer (%s) out of %i computers processes %i of %i pwds\n",
          cname, MCMComputers, MCMThisCycles, MCMTotalCycles);
}


/**
 * Shows fast command-line-options-help and exit
 */
void Help ()
{
  int   I;
  static char *helptext[] =
  {
    "\nASMCRACK [-option1] [-option2] [...]",
    "\t-ACcuracy:num    set the Smart Force accuracy value (default:20)",
    "\t-ALert:num       set the pwd-found-signal length to \"num\" seconds",
    "\t-Bruteforce      use Brute Force word generation",
    "\t-CRacked         show all cracked pwds and exit",
    "\t-CHars:str       use all characters from \"str\" for Brute Force",
    "\t                 (default:abcdefghijklmnopqrstuvwxyz)",
    "\t-DICtfile:name   specify a dictionary file",
    "\t-DISplay         display status approx. every 250th crypt - SLOW!",
    "\t-Geco            create words by login and geco field of the pwd file",
    "\t                 entry, and try this pwds only on this account",
    "\t-HElp            show this screen",
    "\t-List:name       store a cracked-pwds list to \"name\" and exit",
    "\t-MASk:mask       mask Smart Force words (e.g. g___t to find guest)",
    "\t-MAXlength:num   set max. length words to try (default:8)",
    "\t-MInlength:num   set min. length words to try (default:2)",
    "\t-MUpdate         Multi Computer Mode: update account list with",
    "\t                 cracked-pwds-file every 5 minutes",
    "\t-MUlti:name,num  Multi Computer Mode: \"name\": name of config. file",
    "\t                 \"num\": number of this computer (0= 1st, 1= 2nd...)",
    "\t-NONcracked      display non-cracked accounts",
    "\t-NORrestore      don't store restore-information when aborted",
    "\t-Pwdfile:name    specify a linux pwd file",
    "\t-REstore         restore last aborted session by the restore-file",
    "\t-RStfile:name    specify a restore-file (default:\"asmcrack.rst\")",
    "\t-SMartforce      use Smart Force word generation",
    "\t-STdin           read dictionary from stdin",
    "\t-SYllables:num   Syllable Word Generation, \"num\" syllables per word",
    "\t-User:list       only load accounts of the users in \"list\"",
    "\t                 (\"list\" contains logins, separated by commas)",
    "\t-TEst            test the cracking speed by random word generation",
    "\t-TImeabort:num   Abort the session after \"num\" minutes",
    "\t-VDabort:num     Abort the session when v/d-value is less than \"num\".",
    "\t                 Checking is started, when one pwd was cracked.",
    "\t-Words           use dictionary words as pwds (is done by default)",
    "\t-Xuser:list      don't load accounts of the users in \"list\"",
    ""
  };

  for (I = 0; helptext[I][0]; I++)
    puts (helptext[I]);
}

/**
 * MAIN - processes the command line parameters.
 */

int main (unsigned argn, char *argv[])
{
  /*----------------------------------------------------------------------*/
  /* compares the argument n with str, where the argument must be at least
     chrs characters long */
  int ArgCmp (int n, char *str, char chrs)
  {
    int I, c;

    for (I = 1; (c = tolower (argv[n][I])) == tolower (*str++) &&
                c != '\0' && c != ':'; I++);

    if ((c == '\0' || c == ':') && I > chrs)
      return (1);
    return (0);
  }

  void Invalid (int an)
  {
    fprintf (stderr, "\nInvalid argument: \"%s\"\n", argv[an]);
    Help ();
    exit (EX_ERROR);
  }

  /** Reads a string from the argument argv[an] **/
  char *ReadArgStr (char *dest, int an)
  {
    char *col;

    if (!(col = strchr (argv[an], ':')))
      Invalid (an);
    strcpy (dest, col+1);

    return (dest);
  }

  /** Reads an integer from the argument argv[an] **/
  long ReadArgInt (int an, long min, long max)
  {
    char *col;
    long v;

    if (!(col = strchr (argv[an], ':')) ||
        !sscanf (col+1, "%li", &v) || v < min || v > max)
      Invalid (an);

    return (v);
  }

  /** Returns, whether a dictionary is required */
  __inline__ int IsDictRequired ()
  {
    return (!ShowCracked && !ShowNonCracked && DictRequired[CrackMode]);
  }
  /*----------------------------------------------------------------------*/

  register int I,
               c;
  int          restore = 0;     /* restore session */
  char         str[256];

  signal (SIGINT, BreakFunc);

  puts ("\n\t--------------------------------------------------------------");
  puts (  "\t    ASMCrack Version "VERSION_STR" Copyright (C) 1998 David Khling    ");
  puts (  "\t--------------------------------------------------------------");
  puts (  "\t ASMCrack comes with ABSOLUTELY NO WARRENTY. It is free       ");
  puts (  "\t Software and may be distributed under the terms of the       ");
  puts (  "\t GNU General Public License; see gpl.txt for details.         ");
  puts (  "\t If you didn't receive this file, write to the Free Software  ");
  puts (  "\t Foundation, Inc., 59 Temple Place - Suite 330, Boston,       ");
  puts (  "\t MA 02111-1307, USA.                                          ");
  puts (  "\t--------------------------------------------------------------\n");

  Info ();      /* print subversion info from the external crypt module */

  /* process command line options */
  for (I = 1; I < argn; I++)
  {
    if ((c = argv[I][0]) == '/' || c == '-')
    {
      if (ArgCmp (I, "ACuracy",    2))
        SmartForceAcc = ReadArgInt (I, 1, 255);
      else if (ArgCmp (I, "ALert",      2))
        AlertLength = ReadArgInt (I, 0, 255);
      else if (ArgCmp (I, "Bruteforce", 1))
        CrackMode = BRUTE_FORCE;
      else if (ArgCmp (I, "CRacked",    2))
        ShowCracked = 1;
      else if (ArgCmp (I, "CHars",      2))
        ReadArgStr (BFChars, I);
      else if (ArgCmp (I, "DICtfile",   3))
        ReadArgStr (DictName, I);
      else if (ArgCmp (I, "DISplay",    3))
        Display = 1;
      else if (ArgCmp (I, "Geco",   1))
        CrackMode = GECO;
      else if (ArgCmp (I, "HElp",       1))
        Help (), exit (EX_FINISHED);
      else if (ArgCmp (I, "List",       2))
        StorePwdList (ReadArgStr (PwdListName, I));
      else if (ArgCmp (I, "MASk",       3))
        ReadArgStr (Mask, I);
      else if (ArgCmp (I, "MAXlength",  3))
        MaxLength = ReadArgInt (I, 1, 8);
      else if (ArgCmp (I, "MInlength",  2))
        MinLength = ReadArgInt (I, 1, 8);
      else if (ArgCmp (I, "MUpdate",    2))
        MCMUpdate = 1;
      else if (ArgCmp (I, "MUlti",      2))
        ReadArgStr (MCMOptionStr, I);
      else if (ArgCmp (I, "NONcracked", 3))
        ShowNonCracked = 1;
      else if (ArgCmp (I, "NORestore",  3))
        NoRestore = 1;
      else if (ArgCmp (I, "Pwdfile",    1))
        ReadArgStr (PwdName, I);
      else if (ArgCmp (I, "REstore",    2))
        restore = 1;
      else if (ArgCmp (I, "RStfile",    2))
        ReadArgStr (RstName,  I);
      else if (ArgCmp (I, "SMartforce", 2))
        CrackMode = SMART_FORCE;
      else if (ArgCmp (I, "STdin",      2))
        strcpy (DictName, ".stdin");
      else if (ArgCmp (I, "SYllables",  2))
        CrackMode = SYLLABLES, WordSylls = ReadArgInt  (I, 1, 6);
      else if (ArgCmp (I, "Test",       2))
        CrackMode = TEST;
      else if (ArgCmp (I, "Timeabort",  2))
        AbortTime = ReadArgInt (I, 0, 0x7FFFFFFF);
      else if (ArgCmp (I, "User",       1))
        ReadArgStr (UserList, I);
      else if (ArgCmp (I, "Vdabort",    1))
        AbortVD = ReadArgInt (I, 0, 0x7FFFFFFF);
      else if (ArgCmp (I, "Words",      1))
        CrackMode = WORDS;
      else if (ArgCmp (I, "Xuser",      1))
        ReadArgStr (XUserList, I);
      else
        Invalid (I);
    }
    else
      Invalid (I);
  }

  /*** check arguments... ***/

  if (restore)
  {
    if (MinLength != DEFAULT || MaxLength != DEFAULT ||
        SmartForceAcc != DEFAULT || CrackMode != DEFAULT ||
        PwdName[0] || DictName[0] || UserList[0] || XUserList[0] || MCMOptionStr[0])
      fputs ("Warning: Options are overridden by the restore-file.\n", stderr);

    ProcessRstFile ();
  }

  if (Mask[0])
  {
    if (MaxLength != DEFAULT || MinLength != DEFAULT)
      fputs ("Warning: Length specifiers -minlength and -maxlength are overridden by -mask.", stderr);

    if ((MaxLength = MinLength = strlen (Mask)) < 1 || MaxLength > 8)
    {
      fprintf (stderr, "\nInvalid Mask: %s\n\n", Mask);
      exit (EX_ERROR);
    }
  }

  if (MinLength != DEFAULT && MinLength > MaxLength)
  {
    fprintf (stderr, "\nInvalid length (max:%i < min:%i)\n\n", MaxLength, MinLength);
    exit (EX_ERROR);
  }

  if (CrackMode == DEFAULT)
    CrackMode = DEFAULT_CrackMode;

  if (CrackMode != SMART_FORCE && SmartForceAcc != DEFAULT)
    fputs ("Warning: Accuracy value is useless without Smart Force\n", stderr);
  if (CrackMode != BRUTE_FORCE && BFChars[0])
    fputs ("Warning: -chars option is useless without Brute Force\n", stderr);
  if (MCMUpdate && !MCMTotalCycles)
    fputs ("Warning: -mupdate option is useless without -multi\n", stderr);

  printf ("Pwdfile  : ");
  if (!PwdName[0])
    scanf ("%s", PwdName);
  else
    printf ("%s\n", PwdName);

  /* open dictionary file, if required */
  if (IsDictRequired ())
  {
    printf ("Dictfile : ");
    if (!DictName[0])
      scanf ("%s", DictName);
    else
      printf ("%s\n", DictName);
  
    ProcessDictFile (DictName);
  }
  else if (DictName[0])
    fputs ("Warning: Dictionary file is not required in this mode.\n", stderr);

  putchar ('\n');

  if (MCMOptionStr[0])
    if (CrackMode == GECO)
      fputs ("Warning: MCM not supported for geco cracking.\n", stderr);
    else
      ProcessMCMOptions (MCMOptionStr);

  /* Initialize data that are needed by ProcessPwdFile and Crypt*/
  Init_DES ();
  ProcessPwdFile (PwdName);

  if (MinLength == DEFAULT)
    MinLength = DEFAULT_MinLength;
  if (MaxLength == DEFAULT)
    MaxLength = DEFAULT_MaxLength;

  /* Ask whether to overwrite the restore file */
  if (!restore && !NoRestore && __file_exists (RstName) && CrackMode != TEST)
  {
    printf ("\nRestore file \"%s\" allready exists. Overwrite (Y/N)? ", RstName);
    scanf ("%s", str);
    if (stricmp (str, "y"))
      exit (EX_ABORTED);
  }

  I = clock ();         /* initialize clocks to 0 */
  switch (CrackMode)
  {
    case GECO:
      GecoCrack ();
      break;
    case WORDS:
      if (Mask[0])
        fputs ("Warning: Mask not supported for dictionary word cracking.\n", stderr);
      WordCrack ();
      break;
    case SYLLABLES:
      if (Mask[0])
        fputs ("Warning: Mask not supported for Syllable Word Generation.\n", stderr);
      if (Accounts < 10)
        fputs ("Warning: Syllable Word Generation works ineffectively with so few accounts.\n", stderr);
      SyllableCrack ();
      break;
    case SMART_FORCE:
      if (SmartForceAcc == DEFAULT)
        SmartForceAcc = DEFAULT_SmartForceAcc;
      SmartForceCrack ();
      break;
    case BRUTE_FORCE:
      if (!BFChars[0])
        strcpy (BFChars, DEFAULT_BFChars);
      BruteForceCrack ();
      break;
    case TEST:
      SpeedTestCrack ();
      break;
  }

  CrackMode = FINISHED;
  StoreStatus ();

  ShortSignal ();
  return (EX_FINISHED);
}

