/******
 * Module of AMSCRACK
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 **
 * Contains:
 *   Some conversion functions - pwd to num and num to pwd, and the lookup
 *   table creation routines. - Is needed by ASMCRACK and the crypt modules.
 * Needs:
 *   Crypt.c, for some tables (Char2Int and Int2Char)
 */

/***************************************************************************
 *   the following code is almost exactly taken from Xit's crypt.c         *
 ***************************************************************************/
#define   TO_SIX_BIT(rslt, src) {                     \
      C_block cvt;                                 \
      cvt.b[0] = src;          cvt.b[1] = src>>6;    \
      cvt.b[2] = src>>12;      cvt.b[3] = src>>18;   \
      rslt = (cvt.b32.i0 & 0x3f3f3f3fL) << 2;      \
}

#define   ZERO(d,d0,d1)         d = 0
#define   LOAD(d,d0,d1,bl)      d = (bl).b64
#define   LOADREG(d,d0,d1,s,s0,s1)   d = s
#define   OR(d,d0,d1,bl)         d |= (bl).b64
#define   STORE(s,s0,s1,bl)      (bl).b64 = s
#define   DCL_BLOCK(d,d0,d1)   long long d

#define   LGCHUNKBITS   3
#define   CHUNKBITS   (1<<LGCHUNKBITS)
#define   PERM6464(d,d0,d1,cpp,p)            \
   LOAD(d,d0,d1,(p)[(0<<CHUNKBITS)+(cpp)[0]]);      \
   OR (d,d0,d1,(p)[(1<<CHUNKBITS)+(cpp)[1]]);      \
   OR (d,d0,d1,(p)[(2<<CHUNKBITS)+(cpp)[2]]);      \
   OR (d,d0,d1,(p)[(3<<CHUNKBITS)+(cpp)[3]]);      \
   OR (d,d0,d1,(p)[(4<<CHUNKBITS)+(cpp)[4]]);      \
   OR (d,d0,d1,(p)[(5<<CHUNKBITS)+(cpp)[5]]);      \
   OR (d,d0,d1,(p)[(6<<CHUNKBITS)+(cpp)[6]]);      \
   OR (d,d0,d1,(p)[(7<<CHUNKBITS)+(cpp)[7]]);
#define   PERM3264(d,d0,d1,cpp,p)            \
   LOAD(d,d0,d1,(p)[(0<<CHUNKBITS)+(cpp)[0]]);      \
   OR (d,d0,d1,(p)[(1<<CHUNKBITS)+(cpp)[1]]);      \
   OR (d,d0,d1,(p)[(2<<CHUNKBITS)+(cpp)[2]]);      \
   OR (d,d0,d1,(p)[(3<<CHUNKBITS)+(cpp)[3]]);

typedef union {
   unsigned char b[8];
   struct {
      /* long is often faster than a 32-bit bit field */
      long  i0;
      long  i1;
   } b32;
   long long b64;
} C_block;

/* =====  (mostly) Standard DES Tables ==================== */

static unsigned char IP[] = {      /* initial permutation */
   58, 50, 42, 34, 26, 18, 10,  2,
   60, 52, 44, 36, 28, 20, 12,  4,
   62, 54, 46, 38, 30, 22, 14,  6,
   64, 56, 48, 40, 32, 24, 16,  8,
   57, 49, 41, 33, 25, 17,  9,  1,
   59, 51, 43, 35, 27, 19, 11,  3,
   61, 53, 45, 37, 29, 21, 13,  5,
   63, 55, 47, 39, 31, 23, 15,  7,
};

/* The final permutation is the inverse of IP - no table is necessary */

static unsigned char ExpandTr[] = {   /* expansion operation */
   32,  1,  2,  3,  4,  5,
    4,  5,  6,  7,  8,  9,
    8,  9, 10, 11, 12, 13,
   12, 13, 14, 15, 16, 17,
   16, 17, 18, 19, 20, 21,
   20, 21, 22, 23, 24, 25,
   24, 25, 26, 27, 28, 29,
   28, 29, 30, 31, 32,  1,
};

static unsigned char PC1[] = {      /* permuted choice table 1 */
   57, 49, 41, 33, 25, 17,  9,
    1, 58, 50, 42, 34, 26, 18,
   10,  2, 59, 51, 43, 35, 27,
   19, 11,  3, 60, 52, 44, 36,

   63, 55, 47, 39, 31, 23, 15,
    7, 62, 54, 46, 38, 30, 22,
   14,  6, 61, 53, 45, 37, 29,
   21, 13,  5, 28, 20, 12,  4,
};

static unsigned char Rotates[] = {   /* PC1 rotation schedule */
   1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1,
};

/* note: each "row" of PC2 is left-padded with bits that make it invertible */
static unsigned char PC2[] = {      /* permuted choice table 2 */
    9, 18,    14, 17, 11, 24,  1,  5,
   22, 25,     3, 28, 15,  6, 21, 10,
   35, 38,    23, 19, 12,  4, 26,  8,
   43, 54,    16,  7, 27, 20, 13,  2,

    0,  0,    41, 52, 31, 37, 47, 55,
    0,  0,    30, 40, 51, 45, 33, 48,
    0,  0,    44, 49, 39, 56, 34, 53,
    0,  0,    46, 42, 50, 36, 29, 32,
};

static unsigned char S[8][64] = {   /* 48->32 bit substitution tables */
               /* S[1]         */
   14,  4, 13,  1,  2, 15, 11,  8,  3, 10,  6, 12,  5,  9,  0,  7,
    0, 15,  7,  4, 14,  2, 13,  1, 10,  6, 12, 11,  9,  5,  3,  8,
    4,  1, 14,  8, 13,  6,  2, 11, 15, 12,  9,  7,  3, 10,  5,  0,
   15, 12,  8,  2,  4,  9,  1,  7,  5, 11,  3, 14, 10,  0,  6, 13,
               /* S[2]         */
   15,  1,  8, 14,  6, 11,  3,  4,  9,  7,  2, 13, 12,  0,  5, 10,
    3, 13,  4,  7, 15,  2,  8, 14, 12,  0,  1, 10,  6,  9, 11,  5,
    0, 14,  7, 11, 10,  4, 13,  1,  5,  8, 12,  6,  9,  3,  2, 15,
   13,  8, 10,  1,  3, 15,  4,  2, 11,  6,  7, 12,  0,  5, 14,  9,
               /* S[3]         */
   10,  0,  9, 14,  6,  3, 15,  5,  1, 13, 12,  7, 11,  4,  2,  8,
   13,  7,  0,  9,  3,  4,  6, 10,  2,  8,  5, 14, 12, 11, 15,  1,
   13,  6,  4,  9,  8, 15,  3,  0, 11,  1,  2, 12,  5, 10, 14,  7,
    1, 10, 13,  0,  6,  9,  8,  7,  4, 15, 14,  3, 11,  5,  2, 12,
               /* S[4]         */
    7, 13, 14,  3,  0,  6,  9, 10,  1,  2,  8,  5, 11, 12,  4, 15,
   13,  8, 11,  5,  6, 15,  0,  3,  4,  7,  2, 12,  1, 10, 14,  9,
   10,  6,  9,  0, 12, 11,  7, 13, 15,  1,  3, 14,  5,  2,  8,  4,
    3, 15,  0,  6, 10,  1, 13,  8,  9,  4,  5, 11, 12,  7,  2, 14,
               /* S[5]         */
    2, 12,  4,  1,  7, 10, 11,  6,  8,  5,  3, 15, 13,  0, 14,  9,
   14, 11,  2, 12,  4,  7, 13,  1,  5,  0, 15, 10,  3,  9,  8,  6,
    4,  2,  1, 11, 10, 13,  7,  8, 15,  9, 12,  5,  6,  3,  0, 14,
   11,  8, 12,  7,  1, 14,  2, 13,  6, 15,  0,  9, 10,  4,  5,  3,
               /* S[6]         */
   12,  1, 10, 15,  9,  2,  6,  8,  0, 13,  3,  4, 14,  7,  5, 11,
   10, 15,  4,  2,  7, 12,  9,  5,  6,  1, 13, 14,  0, 11,  3,  8,
    9, 14, 15,  5,  2,  8, 12,  3,  7,  0,  4, 10,  1, 13, 11,  6,
    4,  3,  2, 12,  9,  5, 15, 10, 11, 14,  1,  7,  6,  0,  8, 13,
               /* S[7]         */
    4, 11,  2, 14, 15,  0,  8, 13,  3, 12,  9,  7,  5, 10,  6,  1,
   13,  0, 11,  7,  4,  9,  1, 10, 14,  3,  5, 12,  2, 15,  8,  6,
    1,  4, 11, 13, 12,  3,  7, 14, 10, 15,  6,  8,  0,  5,  9,  2,
    6, 11, 13,  8,  1,  4, 10,  7,  9,  5,  0, 15, 14,  2,  3, 12,
               /* S[8]         */
   13,  2,  8,  4,  6, 15, 11,  1, 10,  9,  3, 14,  5,  0, 12,  7,
    1, 15, 13,  8, 10,  3,  7,  4, 12,  5,  6, 11,  0, 14,  9,  2,
    7, 11,  4,  1,  9, 12, 14,  2,  0,  6, 10, 13, 15,  3,  5,  8,
    2,  1, 14,  7,  4, 10,  8, 13, 15, 12,  9,  0,  3,  5,  6, 11,
};

static unsigned char P32Tr[] = {   /* 32-bit permutation function */
   16,  7, 20, 21,
   29, 12, 28, 17,
    1, 15, 23, 26,
    5, 18, 31, 10,
    2,  8, 24, 14,
   32, 27,  3,  9,
   19, 13, 30,  6,
   22, 11,  4, 25,
};

static unsigned char CIFP[] = {      /* compressed/interleaved permutation */
    1,  2,  3,  4,   17, 18, 19, 20,
    5,  6,  7,  8,   21, 22, 23, 24,
    9, 10, 11, 12,   25, 26, 27, 28,
   13, 14, 15, 16,   29, 30, 31, 32,

   33, 34, 35, 36,   49, 50, 51, 52,
   37, 38, 39, 40,   53, 54, 55, 56,
   41, 42, 43, 44,   57, 58, 59, 60,
   45, 46, 47, 48,   61, 62, 63, 64,
};

unsigned char itoa64[] =      /* 0..63 => ascii-64 */
   "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";


/* =====  Tables that are initialized at run time  ==================== */


unsigned char a64toi[128];   /* ascii-64 => 0..63 */

/* Initial key schedule permutation */
C_block   PC1ROT[64/CHUNKBITS][1<<CHUNKBITS] __attribute__ ((aligned (32)));

/* Subsequent key schedule rotation permutations */
C_block   PC2ROT[2][64/CHUNKBITS][1<<CHUNKBITS] __attribute__ ((aligned (32)));

/* Initial permutation/expansion table */
static C_block   IE3264[32/CHUNKBITS][1<<CHUNKBITS] __attribute__ ((aligned (32)));

/* Table that combines the S, P, and E operations.  */
long SPE[2][8][64] __attribute__ ((aligned (32))) __attribute__ ((aligned (32)));

/* compressed/interleaved => final permutation table */
C_block   CF6464[64/CHUNKBITS][1<<CHUNKBITS] __attribute__ ((aligned (32)));

/*
 * Initialize "perm" to represent transformation "p", which rearranges
 * (perhaps with expansion and/or contraction) one packed array of bits
 * (of size "chars_in" characters) into another array (of size "chars_out"
 * characters).
 *
 * "perm" must be all-zeroes on entry to this routine.
 */
void init_perm(perm, p, chars_out)
   C_block perm[64/CHUNKBITS][1<<CHUNKBITS];
   unsigned char p[64];
   int chars_out; {
    int i, j, k, l;

   for (k = 0; k < chars_out*8; k++) {   /* each output bit position */
      l = p[k] - 1;      /* where this bit comes from */
      if (l < 0)
         continue;   /* output bit is always 0 */
      i = l>>LGCHUNKBITS;   /* which chunk this bit comes from */
      l = 1<<(l&(CHUNKBITS-1));   /* mask for this bit */
      for (j = 0; j < (1<<CHUNKBITS); j++) {   /* each chunk value */
         if ((j & l) != 0)
            perm[i][j].b[k>>3] |= 1<<(k&07);
      }
   }
}

/*
 * Initialize various tables.  This need only be done once.  It could even be
 * done at compile time, if the compiler were capable of that sort of thing.
 */
void init_des() {
    int i, j;
    long k;
    int tableno;
   static unsigned char perm[64], tmp32[32];   /* "static" for speed */

   /*
    * table that converts chars "./0-9A-Za-z"to integers 0-63.
    */
   for (i = 0; i < 64; i++)
      a64toi[itoa64[i]] = i;

   /*
    * PC1ROT - bit reverse, then PC1, then Rotate, then PC2.
    */
   for (i = 0; i < 64; i++)
      perm[i] = 0;
   for (i = 0; i < 64; i++) {
      if ((k = PC2[i]) == 0)
         continue;
      k += Rotates[0]-1;
      if ((k%28) < Rotates[0]) k -= 28;
      k = PC1[k];
      if (k > 0) {
         k--;
         k = (k|07) - (k&07);
         k++;
      }
      perm[i] = k;
   }
   init_perm(PC1ROT, perm, 8);

   /*
    * PC2ROT - PC2 inverse, then Rotate (once or twice), then PC2.
    */
   for (j = 0; j < 2; j++) {
      unsigned char pc2inv[64];
      for (i = 0; i < 64; i++)
         perm[i] = pc2inv[i] = 0;
      for (i = 0; i < 64; i++) {
         if ((k = PC2[i]) == 0)
            continue;
         pc2inv[k-1] = i+1;
      }
      for (i = 0; i < 64; i++) {
         if ((k = PC2[i]) == 0)
            continue;
         k += j;
         if ((k%28) <= j) k -= 28;
         perm[i] = pc2inv[k];
      }
      init_perm(PC2ROT[j], perm, 8);
   }

   /*
    * Bit reverse, then initial permutation, then expansion.
    */
   for (i = 0; i < 8; i++) {
      for (j = 0; j < 8; j++) {
         k = (j < 2)? 0: IP[ExpandTr[i*6+j-2]-1];
         if (k > 32)
            k -= 32;
         else if (k > 0)
            k--;
         if (k > 0) {
            k--;
            k = (k|07) - (k&07);
            k++;
         }
         perm[i*8+j] = k;
      }
   }
   init_perm(IE3264, perm, 8);

   /*
    * Compression, then final permutation, then bit reverse.
    */
   for (i = 0; i < 64; i++) {
      k = IP[CIFP[i]-1];
      if (k > 0) {
         k--;
         k = (k|07) - (k&07);
         k++;
      }
      perm[k-1] = i+1;
   }
   init_perm(CF6464, perm, 8);

   /*
    * SPE table
    */
   for (i = 0; i < 48; i++)
      perm[i] = P32Tr[ExpandTr[i]-1];
   for (tableno = 0; tableno < 8; tableno++) {
      for (j = 0; j < 64; j++)  {
         k = (((j >> 0) &01) << 5)|
             (((j >> 1) &01) << 3)|
             (((j >> 2) &01) << 2)|
             (((j >> 3) &01) << 1)|
             (((j >> 4) &01) << 0)|
             (((j >> 5) &01) << 4);
         k = S[tableno][k];
         k = (((k >> 3)&01) << 0)|
             (((k >> 2)&01) << 1)|
             (((k >> 1)&01) << 2)|
             (((k >> 0)&01) << 3);
         for (i = 0; i < 32; i++)
            tmp32[i] = 0;
         for (i = 0; i < 4; i++)
            tmp32[4 * tableno + i] = (k >> i) & 01;
         k = 0;
         for (i = 24; --i >= 0; )
            k = (k<<1) | tmp32[perm[i]-1];
         TO_SIX_BIT(SPE[0][tableno][j], k);
         k = 0;
         for (i = 24; --i >= 0; )
            k = (k<<1) | tmp32[perm[i+24]-1];
         TO_SIX_BIT(SPE[1][tableno][j], k);
      }
   }
}

/****************************************************************************
 ****************************************************************************
 ****************************************************************************/

/*** int 0...63 -> character ***/
/*
char Int2Char[] =
   "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
*/

#define Int2Char itoa64

/*** character -> int 0...63 ***/
/*
char Char2Int[] = {
  ['.']  0, ['/']  1, ['0']  2, ['1']  3, ['2']  4, ['3']  5, ['4']  6, ['5']  7,
  ['6']  8, ['7']  9, ['8'] 10, ['9'] 11, ['A'] 12, ['B'] 13, ['C'] 14, ['D'] 15,
  ['E'] 16, ['F'] 17, ['G'] 18, ['H'] 19, ['I'] 20, ['J'] 21, ['K'] 22, ['L'] 23,
  ['M'] 24, ['N'] 25, ['O'] 26, ['P'] 27, ['Q'] 28, ['R'] 29, ['S'] 30, ['T'] 31,
  ['U'] 32, ['V'] 33, ['W'] 34, ['X'] 35, ['Y'] 36, ['Z'] 37, ['a'] 38, ['b'] 39,
  ['c'] 40, ['d'] 41, ['e'] 42, ['f'] 43, ['g'] 44, ['h'] 45, ['i'] 46, ['j'] 47,
  ['k'] 48, ['l'] 49, ['m'] 50, ['n'] 51, ['o'] 52, ['p'] 53, ['q'] 54, ['r'] 55,
  ['s'] 56, ['t'] 57, ['u'] 58, ['v'] 59, ['w'] 60, ['x'] 61, ['y'] 62, ['z'] 63
};
*/
#define Char2Int a64toi

/**
 * Convert the salt to a number
 */

int ProcessSalt (const char *salt)
{
  int r;
  r = salt[1] == '\0' ? Char2Int['.'] : Char2Int[salt[1]];
  r <<= 8;
  r |= salt[0] == '\0' ? Char2Int['.'] : Char2Int[salt[0]];
  r <<= 2;
  return (r);
}

/**
 * Convert an encrypted pwd-string to a number
 */

long long DES_Str2Num (const char *str)
{
  long m;
  union _N {
    long long     n;
    unsigned char b[8];
  } _n;

  m  = Char2Int[str[0]]; m <<= 6;
  m |= Char2Int[str[1]]; m <<= 6;
  m |= Char2Int[str[2]]; m <<= 6;
  m |= Char2Int[str[3]];
  _n.b[2] = m&0xFF; m >>= 8;
  _n.b[1] = m&0xFF; m >>= 8;
  _n.b[0] = m;
  m  = Char2Int[str[4]]; m <<= 6;
  m |= Char2Int[str[5]]; m <<= 6;
  m |= Char2Int[str[6]]; m <<= 6;
  m |= Char2Int[str[7]];
  _n.b[5] = m&0xFF; m >>= 8;
  _n.b[4] = m&0xFF; m >>= 8;
  _n.b[3] = m;
  m  = Char2Int[str[ 8]]; m <<= 6;
  m |= Char2Int[str[ 9]]; m <<= 6;
  m |= Char2Int[str[10]];
  m >>= 2;
  _n.b[7] = m&0xFF; m >>= 8;
  _n.b[6] = m&0xFF;

  return (_n.n);
}

/**
 * Convert crypt-result to 11 character - string
 */

char *DES_Result2Str (long long n, char *enc)
{
  long m;
  union _N {
    long long     n;
    unsigned char b[8];
  } _n;
  _n.n = n;

  m = ((long)((_n.b[0]<<8) | _n.b[1])<<8) | _n.b[2];
  enc[3] = Int2Char[m&0x3F];   m >>= 6;
  enc[2] = Int2Char[m&0x3F];   m >>= 6;
  enc[1] = Int2Char[m&0x3F];   m >>= 6;
  enc[0] = Int2Char[m&0x3F];
  m = ((long)((_n.b[3]<<8) | _n.b[4])<<8) | _n.b[5];
  enc[3+4] = Int2Char[m&0x3F];   m >>= 6;
  enc[2+4] = Int2Char[m&0x3F];   m >>= 6;
  enc[1+4] = Int2Char[m&0x3F];   m >>= 6;
  enc[0+4] = Int2Char[m&0x3F];
  m = ((long)((_n.b[6])<<8) | _n.b[7])<<2;
  enc[2+8] = Int2Char[m&0x3F];   m >>= 6;
  enc[1+8] = Int2Char[m&0x3F];   m >>= 6;
  enc[0+8] = Int2Char[m&0x3F];

  enc[3+8] = 0;

  return (enc);
}
