/******
 * Module of AMSCRACK
 **
 *   Copyright (C) 1998 David Khling
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *   My Email:   dkuehlin@hell1og.be.schule.de (valid till june 2000)
 *   Snailmail:
 *               David Khling
 *               Lion-Feuchtwanger-Str. 44
 *               12619 Berlin
 *               GERMANY
 **
 * Contains:
 *   All the functions expected by ASMCRACK.
 *   The ASM-Crypt functions, use about 72K of look-up tables, this will run
 *   very fast on computers with slow RAM, but fast Cache.
 * Needs:
 *   Xit's Crypt.c, it contains the function, that creates the tables.
 */


/**
 *  Info - Function, as provided by every of my crypt-modules
 */
.global _Info
        .data
Inf_Str:  .ascii "Subversion: ASM - crypt functions, using 72K lookup tables.\n"
        .text
_Info:
        pushl   $Inf_Str        /* puts ("Version..."); */
        call    _puts
        addl    $4, %esp
        ret

/**
 *  Init_DES - Function, as provided by every of my crypt-modules
 */
.global _Init_DES
_Init_DES:
        call _init_des           /* call crypt.c's init-function */
        ret

/************************************************************************
*************************************************************************
************************************************************************/

        /* DES-Key-Schedule */
        .data
K:      .fill   16, 8, 0

/**
 * DES-Cipher, using the small (72K) look-up tables of crypt.c
 * Arguments:
 *   long long DES_Cipher (long salt)
 */

.global _DES_Cipher

/**
 * When this macro runs, %eax has to be 0
 */
.macro CRUNCH_SMALL a0,a1,b0l,b0,b1l,b1,kn

        /* k = %bp = (b0l ^ b1l) & salt */
        movl    \b0, %ebp
        xorl    \b1, %ebp
        andw    DC_salt, %bp

        /* %edx = B.l[0] = k ^ b0 ^ K[kn].l[0] */
        movl    \b0, %edx
        xorw    %bp, %dx
        xorl    K + \kn*8 + 0, %edx

        /* %ebp = B.b[0] */
        movb    %dl, %al
        /* a0 ^= SPE[0][0][B.b[0]>>2] */
        /* a1 ^= SPE[1][0][B.b[0]>>2] */
        xorl    _SPE + 0 *8*64*4 + 0*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 0*64*4 (%eax), \a1

        /* %ebp = B.b[1] */
        movb    %dh, %al
        shrl    $16, %edx
        /* a0 ^= SPE[0][1][B.b[1]>>2] */
        /* a1 ^= SPE[1][1][B.b[1]>>2] */
        xorl    _SPE + 0 *8*64*4 + 1*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 1*64*4 (%eax), \a1

        /* %ebp = B.b[2] */
        movb    %dl, %al
        /* a0 ^= SPE[0][2][B.b[2]>>2] */
        /* a1 ^= SPE[1][2][B.b[2]>>2] */
        xorl    _SPE + 0 *8*64*4 + 2*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 2*64*4 (%eax), \a1

        /* %ebp = B.b[3] */
        movb    %dh, %al
        /* a0 ^= SPE[0][3][B.b[3]>>2] */
        /* a1 ^= SPE[1][3][B.b[3]>>2] */
        xorl    _SPE + 0 *8*64*4 + 3*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 3*64*4 (%eax), \a1


        /* %edx = B.l[1] = k ^ b1 ^ K[kn].l[1] */
        movl    \b1, %edx
        xorw    %bp, %dx
        xorl    K + \kn*8 + 4, %edx

        /* %ebp = B.b[5] */
        movb    %dl, %al
        /* a0 ^= SPE[0][4][B.b[4]>>2] */
        /* a1 ^= SPE[1][4][B.b[4]>>2] */
        xorl    _SPE + 0 *8*64*4 + 4*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 4*64*4 (%eax), \a1

        /* %ebp = B.b[6] */
        movb    %dh, %al
        shrl    $16, %edx
        /* a0 ^= SPE[0][5][B.b[5]>>2] */
        /* a1 ^= SPE[1][5][B.b[5]>>2] */
        xorl    _SPE + 0 *8*64*4 + 5*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 5*64*4 (%eax), \a1

        /* %ebp = B.b[7] */
        movb    %dl, %al
        /* a0 ^= SPE[0][6][B.b[6]>>2] */
        /* a1 ^= SPE[1][6][B.b[6]>>2] */
        xorl    _SPE + 0 *8*64*4 + 6*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 6*64*4 (%eax), \a1

        /* %ebp = B.b[3] */
        movb    %dh, %al
        /* a0 ^= SPE[0][7][B.b[7]>>2] */
        /* a1 ^= SPE[1][7][B.b[7]>>2] */
        xorl    _SPE + 0 *8*64*4 + 7*64*4 (%eax), \a0
        xorl    _SPE + 1 *8*64*4 + 7*64*4 (%eax), \a1
.endm

        .data
        .balign    4
DC_salt:   .word   0
        .balign    4
DC_i:      .word   0

        .text
.balign 16,0, 8       /* alignment works better on pentiums */
_DES_Cipher:
        movw    4(%esp), %ax    /* high word is allways 0 */
        movw    %ax, DC_salt

        /** ENTER **/
        pushl   %esi
        pushl   %edi
        pushl   %ebx
        pushl   %ebp

        /** L = 0 **/
        xorl    %ebx, %ebx /* low */
        xorl    %ecx, %ecx /* high */
        /** R = 0 **/
        xorl    %esi, %esi /* low */
        xorl    %edi, %edi /* high */

        movw    $12, DC_i  /* i = 12; do {     */
        xorl    %eax, %eax /* %eax = 0; is required by CRUNCH_SMALL */

.balign 16,0x90, 8       /* insert nop's - alignment works better on pentiums */
DC_Loop:
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  0
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  1
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  2
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  3
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  4
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  5
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  6
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  7
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  8
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  9
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 10
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 11
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 12
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 13
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 14
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 15


        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  0
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  1
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  2
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  3
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  4
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  5
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  6
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  7
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  8
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  9
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 10
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 11
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 12
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 13
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 14
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 15

        decw    DC_i          /* }                */
        jnz     DC_Loop       /* while (--i > 0); */

        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  0
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  1
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  2
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  3
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  4
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  5
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  6
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  7
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi,  8
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx,  9
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 10
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 11
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 12
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 13
        CRUNCH_SMALL %ebx, %ecx,   %si, %esi, %di, %edi, 14
        CRUNCH_SMALL %esi, %edi,   %bx, %ebx, %cx, %ecx, 15

        /* R0 = ((R0 >> 3) & 0x0f0f0f0fL) | ((R1 << 1) & 0xf0f0f0f0L); */

        shrl    $3, %esi
        andl    $0x0f0f0f0f, %esi
        shll    $1, %edi
        andl    $0xf0f0f0f0, %edi
        orl     %edi, %esi

        /* R1 = ((L0 >> 3) & 0x0f0f0f0fL) | ((L1 << 1) & 0xf0f0f0f0L); */
        shrl    $3, %ebx
        andl    $0x0f0f0f0f, %ebx
        shll    $1, %ecx
        andl    $0xf0f0f0f0, %ecx
        orl     %ecx, %ebx
        movl    %esi, %ecx
        /** R0 = %ecx; R1 = %ebx **/

        /* Compression, Final Permutation */
        movzbl  %cl, %ebp
        movl    _CF6464 + 0 *256*8 + 0(,%ebp,8), %eax
        movl    _CF6464 + 0 *256*8 + 4(,%ebp,8), %edx
        movzbl  %ch, %ebp
        shrl    $16, %ecx
        orl    _CF6464 + 1 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 1 *256*8 + 4(,%ebp,8), %edx
        movzbl  %cl, %ebp
        orl    _CF6464 + 2 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 2 *256*8 + 4(,%ebp,8), %edx
        movzbl  %ch, %ebp
        orl    _CF6464 + 3 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 3 *256*8 + 4(,%ebp,8), %edx

        movzbl  %bl, %ebp
        orl    _CF6464 + 4 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 4 *256*8 + 4(,%ebp,8), %edx
        movzbl  %bh, %ebp
        shrl    $16, %ebx
        orl    _CF6464 + 5 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 5 *256*8 + 4(,%ebp,8), %edx
        movzbl  %bl, %ebp
        orl    _CF6464 + 6 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 6 *256*8 + 4(,%ebp,8), %edx
        movzbl  %bh, %ebp
        orl    _CF6464 + 7 *256*8 + 0(,%ebp,8), %eax
        orl    _CF6464 + 7 *256*8 + 4(,%ebp,8), %edx

        /** LEAVE **/
        popl    %ebp
        popl    %ebx
        popl    %edi
        popl    %esi
        ret

/**
 * Process DES Key-schedule, using the small (72K) look-up tables of crypt.c
 * Arguments:
 *   void Process_DES_Key (const char *key)
 * NOTE: empty keys ("") will cause problems
 *       it doesn't use %esi. when it uses it, it would have to be saved
 *       in the ENTER - section
 */

.global _Process_DES_Key

.macro  PC2ROTPERM s0l, s0h, s0, s1l, s1h, s1, d0, d1, shift
        movzbl  \s0l, %ebp
        movl    _PC2ROT + (\shift-1)*8*256*8 + 0*256*8 + 0(,%ebp,8), \d0
        movl    _PC2ROT + (\shift-1)*8*256*8 + 0*256*8 + 4(,%ebp,8), \d1
        movzbl  \s0h, %ebp
        shrl    $16, \s0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 1*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 1*256*8 + 4(,%ebp,8), \d1
        movzbl  \s0l, %ebp
        orl     _PC2ROT + (\shift-1)*8*256*8 + 2*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 2*256*8 + 4(,%ebp,8), \d1
        movzbl  \s0h, %ebp
        orl     _PC2ROT + (\shift-1)*8*256*8 + 3*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 3*256*8 + 4(,%ebp,8), \d1

        movzbl  \s1l, %ebp
        orl     _PC2ROT + (\shift-1)*8*256*8 + 4*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 4*256*8 + 4(,%ebp,8), \d1
        movzbl  \s1h, %ebp
        shrl    $16, \s1
        orl     _PC2ROT + (\shift-1)*8*256*8 + 5*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 5*256*8 + 4(,%ebp,8), \d1
        movzbl  \s1l, %ebp
        orl     _PC2ROT + (\shift-1)*8*256*8 + 6*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 6*256*8 + 4(,%ebp,8), \d1
        movzbl  \s1h, %ebp
        orl     _PC2ROT + (\shift-1)*8*256*8 + 7*256*8 + 0(,%ebp,8), \d0
        orl     _PC2ROT + (\shift-1)*8*256*8 + 7*256*8 + 4(,%ebp,8), \d1

        movl    \d0, \s0
        andl    $0xFCFCFCFC, \s0
        movl    \s0, (%esi)
        movl    \d1, 4(%esi)
        addl    $8, %esi
.endm

        .data
DebugStr: .ascii "%11ld, %11ld\n\x00"
        .text
.balign 16,0, 8       /* alignment works better on pentiums */
_Process_DES_Key:

        movl    4(%esp), %eax   /* load key */
        /** ENTER **/
        pushl   %esi
        pushl   %ebx
        pushl   %ebp
        movl    %eax, %esi      /* %esi = key */

        /* %ebx = r.l[0] */
        /* %ecx = r.l[1] */

        cld
        xorl    %eax, %eax
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        movl    _PC1ROT + 0*256*8 + 0(,%eax,8), %ebx
        movl    _PC1ROT + 0*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 1*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 1*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 2*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 2*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 3*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 3*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 4*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 4*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 5*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 5*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 6*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 6*256*8 + 4(,%eax,8), %ecx
        lodsb
        orb     %al, %al
        jz      PDK_StrDone
        shlb    $1, %al
        orl    _PC1ROT + 7*256*8 + 0(,%eax,8), %ebx
        orl    _PC1ROT + 7*256*8 + 4(,%eax,8), %ecx

PDK_StrDone:

        movl    $K, %esi        /* %esi = &K[0] */
        movl    %ebx, %eax
        andl    $0xFCFCFCFC, %eax
        movl    %eax, (%esi)
        movl    %ecx, 4(%esi)
        addl    $8, %esi

        /* Rotates:  1, 1, 2, 2, 2, 2, 2, 2, 1, 2, 2, 2, 2, 2, 2, 1 */
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 1
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 2
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 2
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 2
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 2
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 2
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 2
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 1
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 2
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 2
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 2
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 2
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 2
        PC2ROTPERM %al, %ah, %eax, %dl, %dh, %edx, %ebx, %ecx, 2
        PC2ROTPERM %bl, %bh, %ebx, %cl, %ch, %ecx, %eax, %edx, 1

        /** LEAVE **/
        popl    %ebp
        popl    %ebx
        popl    %esi
        ret
/**
 * DES-Crypts the string >key<, using the salt >salt<, and returns the
 * result as long long number.
 * If key == NULL, it assumes, that the key is the same as it was the last
 * call to Crypt, and so doesn't have process the key-schedule
 * Arguments:
 *   long long Crypt (const char *key, long salt)
 */

.global _Crypt
_Crypt:
        movl     8(%esp), %eax   /* %eax = salt; */
        movl     4(%esp), %ecx   /* %ecx = key; */
        orl     %ecx, %ecx
        jz      C_NoKey
        pushl   %eax             /* save salt for DES_Cipher */
        pushl   %ecx             /* Process_DES_Key (key) */
        call    _Process_DES_Key
        addl    $4, %esp
        popl    %eax             /* salt restored */
C_NoKey:

        pushl   %eax             /* return (DES_Cipher (salt)); */
        call    _DES_Cipher
        addl    $4, %esp

        ret



