(*
gf_files.pas, support unit
22-March-1999

Copyright (c) 1999 General Frenetics, Discorp. All rights reserved.

Author:     marek jedlinski; General Frenetics, Discorp.
E-mail:     <eristic@lodz.pdi.net>
Web URL:    http://www.lodz.pdi.net/~eristic/free/index.html


--------------------------------------------------------------------------

LICENSE, TERMS OF USE

This program is free software; it can be freely distributed and/or
modified under the terms of the `Artistic License' or the `GNU
General Public Licence', described in the two files `ARTISTIC.TXT'
and `GPL.TXT' respectively, which you should have received with the
program package. If you find these files are missing, I will be happy
to send them to you via email. You can also obtain a copy of the GNU
General Public License by writing to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The distribution method is intended to be compliant with the Open Source
Definition -- see: http://www.opensource.org

The software and all accompanying documentation are provided "as is",
and come with ABSOLUTELY NO WARRANTY of any kind, either express or
implied, without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. If you use the contents of this distribution,
you do so at your own risk, and you agree to free the author(s) of any
consequences arising from such use, either intended or otherwise.

--------------------------------------------------------------------------
I hope you find this useful. Comments, suggestions and bug reports are
welcome, see my email address or home page, given on top of this document.

marek jedlinski
General Frenetics, Discorp.
--------------------------------------------------------------------------
*)

unit gf_files;

interface

uses sysutils, gf_const;

const
   // set of characters accepted in filenames (very restrictive)
   FileNameChars  = ['a'..'z', 'A'..'Z', '0'..'9', '_', '-'];
   // typical filters for open/save dialog boxes
   filterTextHTML = 'Text files (*.txt)|*.txt|HTML files (*.htm*)|*.htm*|All files|*.*';
   filterText     = 'Text files (*.txt)|*.txt|All files|*.*';
   filterHTML     = 'HTML files (*.htm*)|*.htm*|All files|*.*';

type
   TFileHandlerProc = procedure(
       const DirInfo : TSearchRec;
       const Path : string );

function ValidateFilename( const s : string; const maxlen : integer; const HasExt : boolean ) : string;
function RandomFileName( const p : string; const ext: string12; len : byte ) : string;

function FilesFound(
   const FileMask : string;  // file name or mask to look for
   const AttrMask : integer; // attributes
   const StartPath : string; // folder to start searchin in
   const Recurse : boolean;  // descend into subfolders?
   const FindAll : boolean;  // find all files ro finish on 1st match
   Handler : TFileHandlerProc ) : integer;


implementation

function ValidateFilename( const s : string;
                           const maxlen : integer;
                           const HasExt : boolean ) : string;
var
   i : integer;
   r : string;
   ext : string;
begin

   ext := '';
   if HasExt then
   begin
       if ( pos( '.', s ) = 0 ) then
       begin
           r := s;
       end
       else
       begin
       for i := length( s ) downto 1 do
           begin
               if ( s[i] <> '.' ) then
               begin
                   ext := s[i] + ext;
               end
               else
               begin
                   ext := '.' + ext;
                   r := copy( s, 1, i-1 );
                   break;
               end;
           end;
       end;
   end
   else
   begin
       r := s;
   end;

   // replace invalid characters with underscores
   for i := 1 to length( r ) do
   begin
       if ( not ( r[i] in FileNameChars )) then
           r[i] := '_';
   end;

   // compress multiple underscores
   i := pos( '__', r );
   while ( i > 0 ) do
   begin
       delete( r, i, 1 );
       i := pos( '__', r );
   end;

   // trim leading and trailing underscores
   while (( r <> '' ) and ( pos( '_', r ) = 1 )) do
       delete( r, 1, 1 );
   while (( r <> '' ) and ( pos( '_', r ) = length( r ))) do
       delete( r, length( r ), 1 );

   // limit length to maxlen
   // (no limit if maxlen = 0 )
   if (( maxlen > 0 ) and ( length( r ) > maxlen )) then
       delete( r, maxlen+1, length( r ));

   if HasExt then
   begin
       if ( r <> '' ) then
           result := r + ext
       else
           result := ''; // no use having extension by itself
   end
   else
   begin
       result := r;
   end;

end; // ValidateFilename

function RandomFileName( const p : string;
                         const ext: string12;
                         len : byte ) : string;
var
   t : string;
   c : byte;
begin
   result := '';
   if ( len < 1 ) then
       len := 8;
   repeat
       t := '';
       for c := 1 to len do
       begin
           t := t + chr( random( 25 ) + 97 ); // #97 to #122
       end;
   until ( not FileExists( p + '\' + t + ext )); // [x] BACKSLASH????!!!

   result := t + ext; // return only filename, without the path

end; // RandomFileName;

function FilesFound(
   const FileMask : string;
   const AttrMask : integer;
   const StartPath : string;
   const Recurse : boolean;
   const FindAll : boolean;
   Handler : TFileHandlerProc ) : integer;
var
   ThisPath : string;
   DirInfo : TSearchRec;
   FileCount, TmpCount : integer;
   FindResult : integer;
begin
   FileCount := 0;
   if ( StartPath = '' ) then
       getdir( 0, ThisPath )
   else
       ThisPath := StartPath;
   if ( ThisPath[length(ThisPath)] <> '\' ) then
       ThisPath := ThisPath + '\';

   FindResult := findfirst( ThisPath+FileMask, AttrMask, DirInfo );
   repeat
       if ( FindResult = 0 ) then // OK, found a file
       begin
           inc( FileCount );
           if assigned(Handler) then
               Handler( DirInfo, ThisPath );
       end;

       if ( not FindAll ) then
           break
       else
           FindResult := FindNext( DirInfo );

   until ( FindResult <> 0 );

   if ( not Recurse ) then
   begin
       result := FileCount;
       findclose( DirInfo );
       exit; // all done
   end;

   FindResult := findfirst( ThisPath+'*.*', faDirectory, DirInfo );
   repeat
       if ( FindResult = 0 ) then
       begin
           if ((( DirInfo.Attr and faDirectory ) > 0 ) and
                ( DirInfo.Name <> '.' ) and ( DirInfo.Name <> '..' )) then
           begin
               TmpCount := FilesFound( FileMask, AttrMask, ThisPath + DirInfo.Name, true, FindAll, Handler );
               if ( TmpCount > 0 ) then
               begin
                   inc( FileCount, TmpCount );
                   if ( not FindAll ) then
                       break;
               end;
           end;
       end;
       FindResult := findnext( DirInfo );
   until ( FindResult <> 0 );

   findclose( DirInfo );
   result := FileCount;
end; // FilesFound

BEGIN
   randomize;
END.
