(*
GUIDClean, version 1.0
22-March-1999

Copyright (c) 1999 General Frenetics, Discorp. All rights reserved.

Author:     marek jedlinski; General Frenetics, Discorp.
E-mail:     <eristic@lodz.pdi.net>
Web URL:    http://www.lodz.pdi.net/~eristic/free/index.html


--------------------------------------------------------------------------

LICENSE, TERMS OF USE

This program is free software; it can be freely distributed and/or
modified under the terms of the `Artistic License' or the `GNU
General Public Licence', described in the two files `ARTISTIC.TXT'
and `GPL.TXT' respectively, which you should have received with the
program package. If you find these files are missing, I will be happy
to send them to you via email. You can also obtain a copy of the GNU
General Public License by writing to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The distribution method is intended to be compliant with the Open Source
Definition -- see: http://www.opensource.org

The software and all accompanying documentation are provided "as is",
and come with ABSOLUTELY NO WARRANTY of any kind, either express or
implied, without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. If you use the contents of this distribution,
you do so at your own risk, and you agree to free the author(s) of any
consequences arising from such use, either intended or otherwise.

--------------------------------------------------------------------------
I hope you find this useful. Comments, suggestions and bug reports are
welcome, see my email address or home page, given on top of this document.

marek jedlinski
General Frenetics, Discorp.
--------------------------------------------------------------------------
*)

unit guid_main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, RXCtrls, Mask, ToolEdit, ComCtrls, FileCtrl, gf_files;

const
   Program_Name    = 'GUIDClean';
   Program_Version = '1.0';
   Program_Credit  = 'General Frenetics, Discorp., 1999';
   Program_License = 'Freeware (GPL)';
   Program_URL     = 'http://www.lodz.pdi.net/~eristic/free/index.html';
   Program_Mailto  = 'eristic@lodz.pdi.net';

   PID_GUID = '_PID_GUID';
   GUID_STRING_SIZE = 36;
   // LEFT_BRACE_POS = 27;
   // RIGHT_BRACE_POS = 101;
   GUID_DATA_SIZE = GUID_STRING_SIZE*2;
   GUID_NULL : array[1..GUID_STRING_SIZE] of char = '00000000-0000-0000-0000-000000000000';
   BUF_SIZE = 4096;
   // 2fac1234-31f8-11b4-a222-08002b34c003

type
  TForm_Main = class(TForm)
    GroupBox1: TGroupBox;
    FolderList: TDirectoryEdit;
    CheckBox_Subfolders: TCheckBox;
    FileList: TTextListBox;
    Label1: TLabel;
    CB_FileTypes: TComboBox;
    Button_AddFiles: TButton;
    Button_Clear: TButton;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    Button_Execute: TButton;
    Label3: TLabel;
    Label_GUID: TLabel;
    CB_SetGUID: TComboBox;
    Button_About: TButton;
    Button_Exit: TButton;
    SBar: TStatusBar;
    OpenDlg: TOpenDialog;
    CheckBox_Backup: TCheckBox;
    CheckBox_Log: TCheckBox;
    Button_CopyGUID: TButton;
    Label4: TLabel;
    Button_AddFolder: TButton;
    Button1: TButton;
    Button_Help: TButton;
    CheckBox_Random: TCheckBox;
    Button_GenerateRandom: TButton;
    procedure Button_ExitClick(Sender: TObject);
    procedure Button_ClearClick(Sender: TObject);
    procedure Button_AddFilesClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RB_FolderClick(Sender: TObject);
    procedure RB_FilesClick(Sender: TObject);
    procedure Button_ExecuteClick(Sender: TObject);
    procedure FileListClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure CB_SetGUIDEnter(Sender: TObject);
    procedure CB_SetGUIDExit(Sender: TObject);
    procedure Button_CopyGUIDClick(Sender: TObject);
    procedure Button_AddFolderClick(Sender: TObject);
    procedure FolderListExit(Sender: TObject);
    procedure FolderListChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button_GenerateRandomClick(Sender: TObject);
    procedure CheckBox_RandomClick(Sender: TObject);
    procedure Button_AboutClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FileListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure Button_HelpClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    UndoString : string;
    IsRunning : boolean;
    AbortRequest : boolean;
    procedure SetState;
    procedure ChangeGUID;
    procedure ShowFileGUID;
    procedure AddFilesFromFolder;
    procedure SetStatusBar;
    procedure TestFiles;
    procedure ShowAbout;
    function AbortRequestOK : boolean;
    procedure ClearSelectedFiles;
    procedure ClearAllFiles;
    procedure AddFiles;
  end;

function GetFileGUID( const fn : string; var guid : string; var guidpos : integer ) : boolean;
function ValidGUIDString( const guid : string ) : boolean;
procedure AddFoundFile( const Dirinfo : TSearchrec; const Path : string );
function GenerateRandomGUID : string;

var
  Form_Main: TForm_Main;
  scan_buf : array[1..BUF_SIZE] of char;
  GUID_Data_Buf : array[1..GUID_DATA_SIZE] of char;
  TempList : TStringList;

implementation

uses guid_about;

{$R *.DFM}

procedure TForm_Main.FormCreate(Sender: TObject);
begin
   Form_Main.Caption := Program_Name + ' ' + Program_Version;
   Application.HelpFile := lowercase( changefileext( ParamStr( 0 ), '.hlp' ));
   CB_FileTypes.ItemIndex := 0;
   Label_GUID.Caption := '(none)';
   CB_SetGUID.MaxLength := GUID_STRING_SIZE;
   CB_SetGUID.Items.Add( GUID_NULL );
   CB_SetGUID.ItemIndex := 0;
   UndoString := '';
   IsRunning := false;
   AbortRequest := false;
   FolderList.Text := ExtractFilePath( ParamStr( 0 ));
   Button_CopyGUID.Enabled := false;
   SetState;
   SetStatusBar;
   Randomize;
end; // CREATE

procedure TForm_Main.Button_ExitClick(Sender: TObject);
begin
   Close;
end;

procedure TForm_Main.Button_ClearClick(Sender: TObject);
begin
   if IsRunning then exit;
   ClearAllFiles;
end;

procedure TForm_Main.Button_AddFilesClick(Sender: TObject);
begin
   if IsRunning then exit;
   AddFiles;
end;

procedure TForm_Main.SetState;
begin
   Button_AddFolder.Enabled := (( FolderList.Text <> '' ) and ( DirectoryExists( FolderList.text )));
end; // SetState

procedure TForm_Main.SetStatusBar;
begin
   Button_Execute.Enabled := ( FileList.Items.Count > 0 );
   SBar.Panels[0].Text := 'Files: ' + inttostr( FileList.Items.Count );
   if ( FileList.Items.Count > 0 ) then
       SBar.Panels[1].Text := 'Ready'
   else
       SBar.Panels[1].Text := 'Please add files to the list.'
end; // SetStatusBar


procedure TForm_Main.RB_FolderClick(Sender: TObject);
begin
   SetState;
end;

procedure TForm_Main.RB_FilesClick(Sender: TObject);
begin
   SetState;
end;

procedure TForm_Main.Button_ExecuteClick(Sender: TObject);
begin
   if IsRunning then exit;
   ChangeGUID;
end;

procedure TForm_Main.ChangeGUID;
var
   cnt, count : integer;
   rnd, log, bak : boolean;
   logf : textfile;
   logfn : string;
   f, bakf : file;
   fn : string;
   guid, newguid : string;
   guidpos : integer;
   errcnt : integer; // files skipped
   proccnt : integer; // files processed correctly
   openresult : integer;
   sizeread : integer;
   wasaborted : boolean;
begin
   errcnt := 0;
   proccnt := 0;
   wasaborted := false;

       if ( FileList.Items.Count = 0 ) then
       begin
           messagedlg( 'No files specified; please add one or more files to the list.', mtError, [mbOK], 0 );
           Button_AddFiles.SetFocus;
           exit;
       end;

   if messagedlg( 'Change GUID information in all listed files?', mtConfirmation, [mbYes,mbNo], 0 ) <> mrYes then exit;

   log := checkbox_log.checked;
   bak := checkbox_backup.checked;
   rnd := CheckBox_Random.checked;

   newguid := CB_SetGUID.Text;
   fillchar( GUID_Data_Buf, GUID_DATA_SIZE, 0 );
   for count := 1 to length( newguid ) do
       GUID_Data_Buf[count*2] := newguid[count];

   if log then
   begin
       logfn := changefileext( Application.Exename, '.log' );
       assignfile( logf, logfn );
       if fileexists( logfn ) then
           append( logf )
       else
           rewrite( logf );
       writeln( logf, 'GUID-Clean start: ' + datetimetostr( now ));
       if rnd then
           writeln( logf, 'Changing document GUID to: (random GUID string)' )
       else
           writeln( logf, 'Changing document GUID to: ', newguid );
       if bak then
           writeln( logf, 'Backup: YES' )
       else
           writeln( logf, 'Backup: NO' );
       writeln( logf );
   end;

   screen.cursor := crHourGlass;
   IsRunning := true;
   for count := 0 to FileList.Items.Count-1 do
   begin
       fn := FileList.Items.Strings[count];
       SBar.Panels[1].Text := '(' + inttostr( count+1 ) + '/' +
                              inttostr( FileList.Items.Count ) + ') ' + fn;
       SBar.Update;
       if GetFileGUID( fn, guid, guidpos ) then
       begin
           assignfile( f, fn );
           {$I-}
           reset( f, 1 );
           openresult := IOResult;
           {$I+}
           if ( openresult <> 0 ) then
           begin
               inc( errcnt );
               if log then
               begin
                   writeln( logf, fn );
                   writeln( logf, #9, 'Cannot write to file: Error ', inttostr( openresult ), ' - file SKIPPED' );
               end;
               continue;
           end;

           if bak then
           begin
               assignfile( bakf, fn + '.bak' );
               {$I-}
               rewrite( bakf, 1 );
               openresult := IOResult;
               {$I+}
               if ( openresult <> 0 ) then
               begin
                   inc( errcnt );
                   if log then
                   begin
                       writeln( logf, fn );
                       writeln( logf, #9, 'Cannot create backup file: Error ', inttostr( openresult ), ' - file SKIPPED' );
                   end;
                   continue;
               end;
               repeat
                   blockread( f, scan_buf, BUF_SIZE, sizeread );
                   blockwrite( bakf, scan_buf, sizeread );
               until ( sizeread <> BUF_SIZE );
               closefile( bakf );
               reset( f, 1 );
           end;
           inc( proccnt );
           seek( f, guidpos{+LEFT_BRACE_POS} );
           if rnd then
           begin
               newguid := GenerateRandomGUID;
               fillchar( GUID_Data_Buf, GUID_DATA_SIZE, 0 );
               for cnt := 1 to length( newguid ) do
                   GUID_Data_Buf[cnt*2] := newguid[cnt];
           end;
           blockwrite( f, GUID_Data_Buf, GUID_DATA_SIZE );
           closefile( f );
           if log then
           begin
               writeln( logf, fn );
               writeln( logf, #9, '- original GUID ', guid, ' - CHANGED' );
           end;
       end
       else
       begin
           inc( errcnt );
           if log then
           begin
               writeln( logf, fn );
               writeln( logf, #9, '- GUID not found, file SKIPPED' );
           end;
       end;

       // check for AbortRequest
       Application.ProcessMessages;
       if AbortRequest then
       begin
           AbortRequest := false;
           wasaborted := true;
           break;
       end;

   end;
   screen.cursor := crDefault;

   if log then
   begin
       writeln( logf, #13#10, 'Total files: ', inttostr( proccnt + errcnt ));
       writeln( logf, 'Files processed : ', inttostr( proccnt ));
       writeln( logf, 'Files skipped   : ', inttostr( errcnt ));
       if wasaborted then
           writeln( logf, 'Execution aborted at user''s request.' );
       writeln( logf, '---------------------------------', #13#10 );
       closefile( logf );
   end;

   {
   MessageDlg( 'Summary:' + #13 +
       'Files processed : ' + inttostr( proccnt ) + #13 +
       'Files skipped   : ' + inttostr( errcnt ) + #13 +
       'Total files     : ' + inttostr( proccnt + errcnt ), mtInformation, [mbOK], 0 );
   }

   SBar.Panels[1].Text := 'Done:' + inttostr( proccnt ) +
                          '  Skipped: ' + inttostr( errcnt ) +
                          '  Total: ' + inttostr( proccnt + errcnt );
   if wasaborted then
       SBar.Panels[1].Text := SBar.Panels[1].Text + ' ABORTED';

   IsRunning := false;
end; // ChangeGUID

procedure TForm_Main.FileListClick(Sender: TObject);
begin
   if IsRunning then exit;
   ShowFileGUID;
end;

procedure TForm_Main.ShowFileGUID;
var
   fn : string;
   guid : string;
   guidpos : integer;
begin
   If ( FileList.ItemIndex < 0 ) then
       exit;
   fn := FileList.Items.Strings[FileList.ItemIndex];

   if GetFileGUID( fn, guid, guidpos ) then
   begin
       label_GUID.Caption := guid;
       Button_CopyGUID.Enabled := true;
   end
   else
   begin
       label_GUID.Caption := '(none)';
       Button_CopyGUID.Enabled := false;
   end;

end; // ShowFileGUID

function GetFileGUID( const fn : string; var guid : string; var guidpos : integer ) : boolean;
var
   f : file;
   openresult : integer;
   sizeread : integer;
   guidp : integer;
   startpos, endpos : integer;
begin
   result := false;
   guidpos := 0;
   guid := '';
   startpos := 0;
   endpos := 0;

   assignfile( f, fn );
   {$I-}
   reset( f, 1 );
   openresult := IOResult;
   {$I+}
   if ( openresult <> 0 ) then
   begin
       messagedlg( 'Error ' + inttostr( openresult ) + ' opening file:' + #13 + fn, mtError, [mbOK], 0 );
       exit;
   end;

   repeat
       fillchar( scan_buf, BUF_SIZE, 0 );
       blockread( f, scan_buf, BUF_SIZE, sizeread );

       guidp := pos( PID_GUID, scan_buf );
       if ( guidp > 0 ) then
       begin
           // file position of the _PID_GUID string:
           guidpos := ( filepos(f) - sizeread ) + (guidp-1);
           seek( f, guidpos );
           // This buffer should contain the whole string now:
           fillchar( scan_buf, BUF_SIZE, 0 );
           blockread( f, scan_buf, BUF_SIZE, sizeread );
           startpos := pos( '{', scan_buf );
           endpos := pos( '}', scan_buf ); // length: 74
           if (( endpos - startpos ) = GUID_DATA_SIZE+2 ) then
           begin
               inc( guidpos, startpos );
               break;
           end
           else
           begin
               guidpos := 0;
           end;
       end;

       if ( sizeread = BUF_SIZE ) then
           seek( f, filepos(f)-100 );

   until ( sizeread <> BUF_SIZE );
   closefile( f );

   if ( guidpos > 0 ) then
   begin
       guid := '';
       for guidp := startpos+1 to endpos-1 do
           if odd( guidp ) then
               guid := guid + scan_buf[guidp];
       // Last check:
       result := ValidGUIDString( guid );
   end;

end; // GetFileGUID

function ValidGUIDString( const guid : string ) : boolean;
var
   x : byte;
   ch : char;
begin
   result := false;
   if ( length( guid ) <> GUID_STRING_SIZE ) then
       exit;
   for x := 1 to GUID_STRING_SIZE do
   begin
       ch := guid[x];
       case x of
           9, 14, 19, 24 : if ( ch <> '-' ) then exit;
           else
           begin
               if ( not( upcase( ch ) in ['0'..'9', 'A'..'F'] )) then exit;
           end;
       end;
   end;
   result := true;

end; // ValidGUIDString

procedure TForm_Main.FormKeyPress(Sender: TObject; var Key: Char);
begin
   case key of
       #27 : begin
           key := #0;
           if IsRunning then
               AbortRequest := AbortRequestOK
           else
               Close;
       end;
   end;
end;

procedure TForm_Main.CB_SetGUIDEnter(Sender: TObject);
begin
   UndoString := CB_SetGUID.Text;
end;

procedure TForm_Main.CB_SetGUIDExit(Sender: TObject);
begin
   if IsRunning then exit;
   if ValidGUIDString( CB_SetGUID.Text ) then
   begin
       if ( CB_SetGUID.Items.IndexOf(CB_SetGUID.Text) < 0 ) then
           CB_SetGUID.Items.Add( CB_SetGUID.Text );
   end
   else
   begin
       case messagedlg( 'The string you entered is not a valid GUID string!' + #13 + CB_SetGUID.Text, mtError, [mbOK,mbCancel], 0 ) of
           mrOK : begin
               CB_SetGUID.SetFocus;
           end
           else
           begin
               if ( CB_SetGUID.Text <> UndoString ) then
                   CB_SetGUID.Text := UndoString
               else
                   CB_SetGUID.ItemIndex := CB_SetGUID.Items.Count-1;
               CB_SetGUID.SetFocus;
           end;

       end;
   end;
end;

procedure TForm_Main.Button_CopyGUIDClick(Sender: TObject);
begin
   if IsRunning then exit;
   if ValidGUIDString( Label_GUID.Caption ) then
   begin
       CB_SetGUID.Items.Add( Label_GUID.Caption );
       CB_SetGUID.ItemIndex := CB_SetGUID.Items.Count-1;
   end;
end;

procedure TForm_Main.Button_AddFolderClick(Sender: TObject);
begin
   if IsRunning then exit;
   AddFilesFromFolder;
end;

procedure TForm_Main.AddFilesFromFolder;
var
   tmpstr : string;
   tmpstr2 : string;
   count : integer;
begin
   case CB_FileTypes.ItemIndex of
       0 : tmpstr := 'MS Word documents';
       1 : tmpstr := 'MS Word templates';
       2 : tmpstr := 'MS Excel spreadsheets';
       3 : tmpstr := 'files';
       else
       begin
           showmessage( 'Choose a file type first!' );
           exit;
       end;
   end;
   if checkbox_subfolders.Checked then
       tmpstr2 := 'and its subdirectories '
   else
       tmpstr2 := '';

   if messagedlg( 'Add all ' + tmpstr +
                  ' in the selected directory ' + tmpstr2 +
                  'to the file list?',
                  mtConfirmation, [mbYes,mbCancel], 0 ) <> mrYes then
       exit;

   TempList := TStringList.Create;
   with TempList do
   begin
       Sorted := true;
       Duplicates := dupIgnore;
       AddStrings( FileList.Items );
   end;

   case CB_FileTypes.ItemIndex of
       0 : tmpstr := '*.doc';
       1 : tmpstr := '*.dot';
       2 : tmpstr := '*.xls';
       3 : tmpstr := '*.*';
   end;

   tmpstr2 := FolderList.text;

   screen.Cursor := crHourGlass;
   count := FilesFound( tmpstr, faAnyFile, tmpstr2, checkbox_subfolders.Checked, true, @AddFoundFile );
   FileList.Clear;
   FileList.Items.AddStrings( TempList );
   TempList.Free;
   SetStatusBar;
   screen.Cursor := crDefault;

   if ( count = 0 ) then
       SBar.Panels[1].text := 'Could not find any matching files'
   else
       SBar.Panels[1].text := 'Added ' + inttostr( count ) + ' files.';

end; // AddFilesFromFolder

procedure AddFoundFile( const Dirinfo : TSearchrec; const Path : string );
begin
   TempList.Add( Path + DirInfo.Name );
end; // AddFoundFile

procedure TForm_Main.FolderListExit(Sender: TObject);
begin
   SetState;
end;


procedure TForm_Main.FolderListChange(Sender: TObject);
begin
   SetState;
end;

procedure TForm_Main.Button1Click(Sender: TObject);
begin
   if IsRunning then exit;
   TestFiles;
end;

procedure TForm_Main.TestFiles;
var
   thelist : TStringList;
   totalcnt, guidpos, cnt, remcnt : integer;
   guid : string;
begin

   if ( FileList.Items.Count = 0 ) then
   begin
       messagedlg( 'File list is empty; nothing to test!', mtInformation, [mbOK], 0 );
       exit;
   end;
   theList := TStringList.Create;
   remcnt := 0;
   totalcnt := FileList.Items.Count;

   screen.Cursor := crHourGlass;

   for cnt := 0 to FileList.Items.Count-1 do
   begin
       if GetFileGUID( FileList.Items.Strings[cnt], guid, guidpos ) then
           theList.Add( FileList.Items.Strings[cnt] )
       else
           inc( remcnt );
   end;

   FileList.Clear;
   FileList.Items.AddStrings( theList );
   SetStatusBar;
   screen.Cursor := crDefault;
   theList.Free;

   messagedlg( 'Tested ' + inttostr( totalcnt ) + ' files.' + #13 +
               'Removed ' + inttostr( remcnt ) + ' files with no GUID data.' + #13 +
               'Kept ' + inttostr( FileList.Items.Count ) + ' documents.', mtInformation, [mbOK], 0 );
end; // TestFiles

procedure TForm_Main.Button_GenerateRandomClick(Sender: TObject);
begin
   if IsRunning then exit;
   CB_SetGUID.Text := GenerateRandomGUID;
end;

function GenerateRandomGUID : string;
var
   s : string;
   r, cnt : byte;
begin
   s := '';
   for cnt := 1 to GUID_STRING_SIZE do
   begin
       r := random( 16 );
       case r of
           0..9 : s := s + chr(r+48);
           else
           begin
               s := s + chr(r+55);
           end;
       end;
   end;
   s[9] := '-';
   s[14] := '-';
   s[19] := '-';
   s[24] := '-';
   result := s;
   if ( not ValidGUIDString( s )) then
   begin
       result := GUID_NULL;
   end;
end; // GenerateRandomGUID


procedure TForm_Main.CheckBox_RandomClick(Sender: TObject);
begin
   if IsRunning then exit;
   CB_SetGUID.Enabled := ( not CheckBox_Random.checked );
end;

procedure TForm_Main.Button_AboutClick(Sender: TObject);
begin
   ShowAbout;
end;

procedure TForm_Main.ShowAbout;
begin
   Form_About := TForm_About.Create(self);
   try
       Form_About.ShowModal;
   finally
       Form_About.Free;
   end;
end; // ShowAbout

procedure TForm_Main.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
   if IsRunning then
   begin
       AbortRequest := AbortRequestOK;
       CanClose := AbortRequest;
   end;
end;

function TForm_Main.AbortRequestOK : boolean;
begin
   result := ( messagedlg(
       'GUIDClean has not finished modifying the files.' + #13 +
       'Are you sure you want to abort the operation?',
       mtConfirmation, [mbNo,mbYes], 0
   ) = mrYes );

end; // AbortRequestOK

procedure TForm_Main.FileListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   if isRunning then exit;
   case key of
       45 : begin // INSERT
           AddFiles;
       end;
       46 : begin
           ClearSelectedFiles;
       end; // DELETE
   end;
end;
procedure TForm_Main.ClearAllFiles;
begin
   if isRunning then exit;
   if ( FileList.Items.Count = 0 ) then exit;
   if ( messagedlg( 'Remove all listed files?', mtConfirmation, [mbYes,mbNo], 0 ) <> mrYes ) then exit;
   FileList.Clear;
   label_GUID.Caption := '';
   Button_CopyGUID.Enabled := false;
   SetStatusBar;
end; // ClearAllFiles

procedure TForm_Main.ClearSelectedFiles;
var
   i, j : integer;
begin
   if isRunning then exit;
   if ( FileList.Items.Count < 1 ) then
       exit;
   if ( FileList.SelCount = 0 ) then
       FileList.Selected[FileList.ItemIndex] := true;

       FileList.Items.BeginUpdate;
       for i := 1 to FileList.SelCount do
       begin
           for j := 0 to FileList.Items.Count-1 do
               if FileList.Selected[j] then
               begin
                   FileList.Items.Delete(j);
                   break;
           end;
       end;
       FileList.Items.EndUpdate;

   label_GUID.Caption := '';
   Button_CopyGUID.Enabled := false;
   SetState;
   SetStatusBar;
end; // ClearSelectedFiles;

procedure TForm_Main.AddFiles;
begin
   if IsRunning then exit;
   if OpenDlg.Execute then
   begin
       FileList.Items.AddStrings( OpenDlg.Files );
       SetStatusBar;
   end;
end; // AddFiles;

procedure TForm_Main.Button_HelpClick(Sender: TObject);
begin
   Application.HelpContext(10);
end;

end.


